#include <iostream>
#include <fstream>
#include <cmath>
#include <cstdlib>
#include <gsl/gsl_rng.h>
#include "logfunc.h"
#include "logstuff.h"

#define DEBUG
#define WCORR
//#define PLOT

using namespace std;

typedef long double prec;

gsl_rng *rng = gsl_rng_alloc(gsl_rng_ran3);
logfactorial<prec, 1024> ln_nfac;

const prec ln_0 = LOGFUNC(prec(0.));
const prec ln_2 = LOGFUNC(prec(2.));

inline double ran() {return gsl_rng_uniform(rng);}

inline int sgn(prec x)
{
  if(x > 0) return 1;
  if(x < 0) return -1;
  return 0;
}

prec ln_binom(int n, int k);

struct mc_step_t
{
  int seq0;
  int seq1;
  int delta;
};

class distr_t
{
  static const int n_beta = 12;
  static const prec beta[n_beta];
  prec g[n_beta];
  prec ln_z[n_beta];
  long long n_saved;
  prec ln_w;
  prec w[n_beta];
#ifdef WCORR
  prec w_corr[n_beta][n_beta];
#endif
  prec ln_z_1_diag;

  prec dg_func(prec dg) /*{return 0.5*dg;}*/ {return (abs(dg)>2)?0.5*dg+sgn(dg):dg;};
public:
  distr_t();
  void z_reset();
  bool g_update();
  prec add(prec s, bool d);
  prec ln_weight(prec s);
  int w_peak();
  prec ln_z_diff(){return g[n_beta-1]+ln_z[n_beta-1]-g[0]-ln_z[0];};
};
const prec distr_t::beta[n_beta] = //{0.,0.,.1,.2,.3,.4,.5,.6,.7,.8,.9,1.};
{.0, .018, .055, .09, 
.13, .18, .28, .42,
.6, .78, .92, 1.};
/*{.0, .001, .005, .02,
//.015, .025, .035, .05,
.05, .1, .15, .25,
.35, .5, .7, 1.};*/

template<int c>
class cycle_t
{
#ifdef DEBUG
public:
#endif
  static const int m = 1 << c - 1;
  int n;
  int n_seq[m];
  long long f[m];
  long long new_f[m];
  prec entropy;
  prec new_entropy;
  
public:
  cycle_t(int nt){restart(nt);}
  void restart(){restart(n);}
  void restart(int nt);
  mc_step_t propose();
  void update(mc_step_t step);
  void update(mc_step_t step, prec s);
  prec eval(mc_step_t step);
  prec calc_entropy();
  prec get_entropy(){return entropy;}
  int get_n(){return n;}
};


template<int c> int shift(int seq);
template<int c> int n_seqgates(int i_seq0, int i_seq1, int o_seq);
template<int c> prec n_cycles(int n_nodes, long long n_mc_steps, int n_therm);
template<int c> prec n_cycles(int n_nodes, long long n_mc_steps, int n_therm,
				distr_t &distr);

int main(int argc, char **argv)
{
  if(argc != 2){
    cerr << "using: n_nodes\n";
    exit(-1);
  }
  int n = atoi(argv[1]);

  const int c = 5;
//  const int max_n = 10000;
  const int n_therm = 10000;
  const int n_mc =    80000;
  gsl_rng_set(rng, 3243515);
//  gsl_rng_set(rng, time(NULL));
  cout << c << " " << n << "\n";
  prec tmp = n_cycles<c>(n, n_mc, n_therm);
  cout << tmp << endl;
}


prec ln_binom(int n, int k)
{
  return ln_nfac(n) - ln_nfac(k) - ln_nfac(n-k);
}

distr_t::distr_t()
{
  for(int i = 0; i < n_beta; i++)
    g[i] = 0.;
  z_reset();
}

void distr_t::z_reset()
{
  ln_z_1_diag = ln_0;
  for(int i = 0; i < n_beta; i++)
    ln_z[i] = ln_0;
  n_saved = 0;  
#ifdef WCORR
  for(int i = 0; i < n_beta; i++)
    for(int j = 0; j < n_beta; j++)
      w_corr[i][j] = 0;
#endif
}

bool distr_t::g_update()
{
#ifdef DEBUG
#ifdef WCORR
  cout << "overlap:";
  for(int i = 1; i < n_beta; i++){
    cout <<" "
	 <<(w_corr[i-1][i] + w_corr[i][i-1])/(w_corr[i-1][i-1] + w_corr[i][i]);
  }
  cout << endl;
#endif
  cout << "ln(z_sample):";
  for(int i = 0; i < n_beta; i++)
    cout << " " << ln_z[i];
  cout << endl;
  cout << "ln(z - z_diag)" << ln_z[n_beta-1] - ln_z_1_diag << endl;
  cout << "dg:";
#endif
  for(int i = 0; i < n_beta; i++){
    prec g_new = g[i]+ln_z[i]-g[0]-ln_z[0];
#ifdef DEBUG
    cout <<" "<<dg_func(g_new - g[i]);
#endif
    g[i] += dg_func(g_new - g[i]);
  }    
#ifdef DEBUG
  cout << endl;
#endif
  return true;
}

prec distr_t::add(prec s, bool is_diag)
{
  static prec w_new[n_beta];

  prec ln_w;
  if(s == ln_0){
    ln_w = g[0];
    w_new[0] = 1.;
    for(int i=1; i < n_beta; i++)
      w_new[i] = 0.;
  }
  else{
    ln_w = ln_weight(s);
    w_new[0] = exp(-g[0] - ln_w);
    for(int i=1; i < n_beta; i++)
      w_new[i] = exp(beta[i]*s - g[i] - ln_w);
  }
  
#ifdef WCORR
  if(n_saved){
    for(int i=0; i < n_beta; i++)
      for(int j=0; j < n_beta; j++)
	w_corr[i][j] += w[i]*w_new[j];
  }
#endif
  for(int i=0; i < n_beta; i++)
    w[i] = w_new[i];
  n_saved++;
  
  ln_z[0] = ln_add(ln_z[0], - g[0] - ln_w);
  if(s == ln_0) return ln_w;
  for(int i=1; i < n_beta; i++)
    ln_z[i] = ln_add(ln_z[i], beta[i]*s - g[i] - ln_w);
  //if(is_diag){
  //  ln_z_1_diag = ln_add(ln_z[i], s - g[i] - ln_w);
  //}
  return ln_w;
}

prec distr_t::ln_weight(prec s)
{
  prec ln_w = g[0];
  if(s == ln_0) return ln_w;
  for(int i=1; i < n_beta; i++)
    ln_w = ln_add(ln_w, beta[i]*s - g[i]);
  return ln_w;
}
int distr_t::w_peak()
{
  prec m=0.;
  int mi = -1;
  for(int i = 0; i < n_beta; i++)
    if(w[i] > m){
      mi = i;
      m = w[i];
    }
  return mi;
}

template<int c>
void cycle_t<c>::restart(int nt)
{
  n = nt;
  //draw a radom m-partitioning of n
  bool split[n+m];

  for(int i = 0; i < n+m; i++)
    split[i] = false;
  split[n+m-1] = true;

  for(int i = 1, j; i < m; i++){
    while(split[j = int((n+m)*ran())]);
    split[j] = true;
  }

  for(int i = 0, j = -1, k = 0; i < n+m; i++)
    if(split[i]){
      n_seq[k++] = i - j - 1;
      j = i;
    }

#ifdef DEBUG
  int s = 0;
  for(int i = 0; i < m; i++)
    s += n_seq[i];
  if(s != n){cerr << "Bug in cycle_t::restart\n"; exit(-1);}
#endif

  calc_entropy();
}

template<int c>
mc_step_t cycle_t<c>::propose()
{
  mc_step_t step;
  step.seq0 = int(ran()*m);
  step.seq1 = int(ran()*(m - 1));
  if(step.seq1 >= step.seq0) step.seq1++;

  step.delta = int(exp(ran()*LOGFUNC(prec(n_seq[step.seq0] + n_seq[step.seq1]))));
  if(n_seq[step.seq1] + step.delta < 0 || n_seq[step.seq0] - step.delta < 0)
    step.delta = 0;
  return step;
}

//template<int c>
// void cycle_t<c>::update(mc_step_t step)
// {
//   if(step.delta){
//     n_seq[step.seq1] += step.delta;
//     n_seq[step.seq0] -= step.delta;
//     calc_entropy();
//   }
// }

template<int c>
void cycle_t<c>::update(mc_step_t step, prec s)
{
  if(step.delta){
    if(s != new_entropy){
      cerr << s << " " << new_entropy << " Buddeligugg!\n";
      exit(-1);
    }
    n_seq[step.seq1] += step.delta;
    n_seq[step.seq0] -= step.delta;
    entropy = s;
    for(int i=0; i<m; i++)
      f[i] = new_f[i];
  }
  else if(entropy != s){
    cerr << "Bubbelibugg!\n";
    exit(-1);
  }
}

template<int c>
prec cycle_t<c>::eval(mc_step_t step)
{
  if(step.delta == 0) return entropy;

  static const prec s0 = ln_nfac(n);
  new_entropy = s0;

  const int d = step.delta;
  const int l1 = step.seq1;
  const int l0 = step.seq0;

  n_seq[l1] += d;
  n_seq[l0] -= d;
  for(int i = 0; i < m; i++){
    new_f[i] = f[i] - d*d*(n_seqgates<c>(l1,l1,i) + n_seqgates<c>(l0,l0,i) -
			   2*n_seqgates<c>(l0,l1,i));
    for(int j = 0; j < m; j++)
      new_f[i] += 2*d*n_seq[j]*(n_seqgates<c>(j,l1,i)-n_seqgates<c>(j,l0,i));

    if(new_f[i] < 0){
      cerr << "Gaaak\n";
      exit(-1);
    }
    if(n_seq[i]){
      new_entropy += LOGFUNC(prec(new_f[i]))*n_seq[i];
      new_entropy -= ln_nfac(n_seq[i]);
    }
  }
  
  if(c == 6){
    if(n_seq[0] + n_seq[21] == n) new_entropy = ln_0;
    if(n_seq[0] + n_seq[9] + n_seq[18] + n_seq[27] == n) new_entropy = ln_0;
  }
  else if(c == 4){
    if(n_seq[0] + n_seq[5] == n) new_entropy = ln_0;
  }
  else{
    if(n_seq[0] == n) new_entropy = ln_0;
  }

  n_seq[l1] -= d;
  n_seq[l0] += d;

  return new_entropy;
}

template<int c>
prec cycle_t<c>::calc_entropy()
{
  entropy = ln_nfac(n);

  for(int i = 0; i < m; i++){
      f[i] = 0;
      for(int l = 0; l < m; l++)
	if(n_seq[l])
	  f[i] += n_seqgates<c>(l,l,i)*n_seq[l]*n_seq[l];
      for(int j = 0; j < m; j++)
	if(n_seq[j])
	  for(int l = 0; l < j; l++)
	    if(n_seq[l])
	      f[i] += 2*n_seqgates<c>(j,l,i)*n_seq[j]*n_seq[l];
      
      if(n_seq[i]){
	entropy += LOGFUNC(prec(f[i]))*n_seq[i];
	entropy -= ln_nfac(n_seq[i]);
      }
  }

  if(c == 6){
    if(n_seq[0] + n_seq[21] == n) return ln_0;
    if(n_seq[0] + n_seq[9] + n_seq[18] + n_seq[27] == n)
        return ln_0;
  }
  else if(c == 4){
    if(n_seq[0] + n_seq[5] == n) return ln_0;
  }
  else{
    if(n_seq[0] == n) return ln_0;
  }
  
  return entropy;
}

template<int c>
int shift(int seq)
{
  if(seq & 1)
    return (seq >> 1) ^ ((1 << c - 1) - 1);
  else
    return seq >> 1;
}

template<int c>
class p8_tensor
{
  static const int c_ = c - 1;
  static const int m = 1 << c_;
  int tensor[m*m*m];
  int p(int i_seq0, int i_seq1, int o_seq){
    o_seq = shift<c>(o_seq);
    return 
      (o_seq == 0) + 
      (o_seq == i_seq0) + 
      (o_seq == i_seq1) +
      (o_seq == (i_seq0 ^ i_seq1)) +
      (o_seq == (i_seq0 & i_seq1)) +
      (o_seq == (i_seq0 | i_seq1)) +
      (o_seq == (~i_seq0 & i_seq1)) +
      (o_seq == (i_seq0 & ~i_seq1));
  }
public:

  p8_tensor(){
    for(int i=0; i<m; i++)
      for(int j=0; j<m; j++)
	for(int k=0; k<m; k++)
	  tensor[(((k << c_) | i) << c_) | j] = p(i,j,k);
  }
  int operator()(int i_seq0, int i_seq1, int o_seq){
    return tensor[(((o_seq << c_) | i_seq0) << c_) | i_seq1];
  }    
};

template<int c>
inline int n_seqgates(int i_seq0, int i_seq1, int o_seq)
{
  static p8_tensor<c> p_distr;
  return p_distr(i_seq0, i_seq1, o_seq);
}

template<int c>
prec n_cycles(int n_nodes, long long n_mc_steps, int n_therm)
{
  distr_t distr;
  for(int i = 0; i < 5; i++){
    prec v = n_cycles<c>(n_nodes, (n_mc_steps << i) + n_therm, n_therm, distr);
    cout << "n cycles: " << v << endl;
  }
  return 0.;
}

template<int c>
prec n_cycles(int n_nodes, long long n_mc_steps, int n_therm, distr_t &distr)
{
#ifdef PLOT
  ofstream plot("plot.dat");
#endif

  cycle_t<c> cycle(n_nodes);
  cycle.restart();
  for(long long i = 0; i < n_therm + n_mc_steps; i++){
#ifdef DEBUG
    if(i%1000000==0) cout << "-" << flush;
#endif
    mc_step_t step = cycle.propose();
#ifdef DEBUG
    if(step.seq0 < 0 || step.seq0 >= 1 << c - 1 ||
       step.seq1 < 0 || step.seq1 >= 1 << c - 1){
      cerr << "Bug: step.seq = " << step.seq0 <<", "<< step.seq1 << endl; exit(-1);
    }
#endif
    prec ln_w_old = (i >= n_therm)? distr.add(cycle.get_entropy(),false):
                                      distr.ln_weight(cycle.get_entropy());
#ifdef PLOT
    if(i%100==0) plot <<i<<"\t"<<distr.w_peak()<<"\t"<<cycle.get_entropy()<<"\t"<<ln_w_old<<endl;    
#endif
    prec s_new = cycle.eval(step);
    if(exp(distr.ln_weight(s_new)-ln_w_old) > ran()){
      cycle.update(step,s_new);
      if(ran()<0.0001 && abs(cycle.calc_entropy()-s_new)>1e-8){
	cerr << "S-discrep: " << s_new-cycle.calc_entropy() << endl;
	exit(-1);
      }
    }
  }

#ifdef PLOT
  plot.close();
#endif

#ifdef DEBUG
  cout << endl;
#endif
  //cout << endl << "-- "<< cycle.n_seq[0] << endl;
  prec ln_z = distr.ln_z_diff();
  ln_z += ln_binom(n_nodes + (1 << c - 1) - 1, n_nodes);
  ln_z -= (3*ln_2 + 2*LOGFUNC(prec(n_nodes)))*n_nodes;
  ln_z -= LOGFUNC(prec(c));

  distr.g_update();
  distr.z_reset();

  return exp(ln_z);
}


