#include <vector>
#include <iostream>
#include <fstream>
#include <cmath>
#include <ctime>
#include <cstdio>
#include <numeric>
#include <map>
#include <set>
#include "gsl_rng.h"
#include "gsl_randist.h"

#include "knet.h"

using namespace std;

gsl_rng *rng = gsl_rng_alloc(gsl_rng_taus);


struct intmore
{
	bool operator()(const int &a, const int &b)
	{
		return a > b;
	}
};

typedef multimap<int, nodeid, intmore> kmap_t;

int networkFromKDistributionsPrepare(int N, const map<int, int> &kfreq,
	const char *what, kmap_t &kmap)
{
	int sum(0), totk(0);
	for(map<int, int>::const_iterator i = kfreq.begin();
		i != kfreq.end(); i++)
	{
		int k = (*i).first;
		int cnt = (*i).second;
		if(sum + cnt > N)
		{
			cerr << "K_" << what << " distribution adds up to more than N\n";
			return -1;
		}
		for(int j = 0; j < cnt; j++)
			kmap.insert(make_pair(k, nodeid(sum + j)));
		sum += cnt;
		totk += cnt * k;
	}
	if(sum != N)
	{
		cerr << "K_" << what << " distribution adds up to less than N\n";
		return -1;
	}
	return totk;
}

bool networkFromKDistributionsChooseNodes(
	kmap_t &kmap1, kmap_t &kmap2, vector<set<nodeid> >&edge,
	vector<set<nodeid> >&invedge)
{
	int cnt =  (*kmap1.begin()).first;
	nodeid n1 = (*kmap1.begin()).second;
	kmap1.erase(kmap1.begin());
	if(cnt > 1)
		kmap1.insert(make_pair(cnt - 1, n1));
	int maxr = 0;
	for(kmap_t::iterator i = kmap2.begin();
		i != kmap2.end(); i++)
	{
		if(edge[n1].find((*i).second) == edge[n1].end())
			maxr += (*i).first;
	}
	if(!maxr)
	{
//		cerr << "Node " << n1 << " unable to connect further\n";
		return false;
	}
	int r = gsl_rng_uniform_int(rng, maxr);
	kmap_t::iterator i;
	for(i = kmap2.begin(); i != kmap2.end(); i++)
	{
		if(edge[n1].find((*i).second) == edge[n1].end() &&
			(r -= (*i).first) < 0)
		{
			break;
		}
	}
	cnt =  (*i).first;
	nodeid n2 = (*i).second;
	kmap2.erase(i);
	if(cnt > 1)
		kmap2.insert(make_pair(cnt - 1, n2));
	edge[n1].insert(n2);
	invedge[n2].insert(n1);
	return true;
}

bool networkFromKDistributions(knet &net, int N, const map<int, int> &kinfreq,
	const map<int, int> &koutfreq, bool picky = true)
{
	vector<nodeid> outremap(N);
	for(int i = 0; i < N; i++)
		outremap[i] = i;
	for(int i = N; i > 1; i--)
	{
		int pos = gsl_rng_uniform_int(rng, i);
		if(pos != i - 1)
			swap(outremap[pos], outremap[i - 1]);
	}

	kmap_t kinmap, koutmap;
	int tmp1 = networkFromKDistributionsPrepare(N, kinfreq, "in", kinmap);
	int tmp2 = networkFromKDistributionsPrepare(N, koutfreq, "out", koutmap);
	if(tmp1 < 0 || tmp2 < 0)
	{
		cerr << "networkFromKDistributions() failed\n";
		return false;
	}
	if(picky && tmp1 != tmp2)
	{
		cerr << "sum(K_in) is not equal to sum(K_out)\n";
		return false;
	}

	vector<set<nodeid> > edge(N), invedge(N);
	while(!kinmap.empty() && !koutmap.empty())
	{
		if((*kinmap.begin()).first >= (*koutmap.begin()).first)
		{
			networkFromKDistributionsChooseNodes(kinmap, koutmap,
				edge, invedge);
		}
		else
		{
			networkFromKDistributionsChooseNodes(koutmap, kinmap,
				invedge, edge);
		}
	}

	net.clear();
	net.resize(N);
	for(int i = 0; i < N; i++)
	{
		set<nodeid> &e = edge[i];
		for(set<nodeid>::iterator j = e.begin(); j != e.end(); j++)
			net[i].inputs.push_back(*j);
	}
	return true;
}

int powerlawGenerate(int N, int totk, double power, map<int, int> &kfreq,
	bool limit_k = true)
{
	const int use0 = 0;
	kfreq.clear();
	vector<double> kprob(N + use0);
	kprob[0] = 1;
	for(int k = 1; k <= N; k++)
		kprob[k - 1 + use0] = pow(double(k), power);

	int kmax = N, kleft = totk, nodes = 0;
	gsl_ran_discrete_t *discr = gsl_ran_discrete_preproc(N + use0, &kprob[0]);

	if(limit_k)
	{
		while(kleft)
		{
			if(kleft < kmax)
			{
				kmax = kleft;
				gsl_ran_discrete_free(discr);
				discr = gsl_ran_discrete_preproc(kmax + use0, &kprob[0]);
			}

			int k = gsl_ran_discrete(rng, discr) + 1 - use0;
			kfreq[k]++;
			kleft -= k;
			nodes++;
		}
		gsl_ran_discrete_free(discr);
		return nodes;
	}
	else	// limit on N instead
	{
		int ksum = 0;
		while(nodes < N)
		{
			int k = gsl_ran_discrete(rng, discr) + 1 - use0;
			kfreq[k]++;
			ksum += k;
			nodes++;
		}
		gsl_ran_discrete_free(discr);
		return ksum;
	}
}

bool powerlawNetwork(knet &net, int N, double mean_k, double power_kin,
	double power_kout, bool requireexact)
{
	int totk = int(mean_k * double(N) + .5);
 	map<int, int> kinfreq, koutfreq;

	if(!requireexact)
	{
		powerlawGenerate(N, totk, power_kin, kinfreq, false);
		powerlawGenerate(N, totk, power_kout, koutfreq, false);
		return networkFromKDistributions(net, N, kinfreq, koutfreq, false);
	}

//	cerr << "Generating distributions: ";
	int i = 100;
	while(powerlawGenerate(N, totk, power_kin, kinfreq, false) !=
		powerlawGenerate(N, totk, power_kout, koutfreq, false))
	{
		if(!--i)
		{
//			cerr << ".";
			i = 100;
		}
	}
//	cerr << "!\n";


	return networkFromKDistributions(net, N, kinfreq, koutfreq);
}

// main() for cycle statistics
int main(int argc, char **argv)
{
	int randseed = 0;

	if(argc != 6 && argc != 7)
	{
		cerr << "usage: makenet nodes archs mingamma deltagamma steps [randseed]\n";
		return 1;
	}

	if(argc == 6)
	{
		ifstream r("/dev/urandom");
		char rbuf[5];
		r.read(rbuf, 4);
		randseed = *(int *)rbuf;
	}
	else
	{
		randseed = strtoul(argv[6], NULL, 0);
	}
//	cerr << "Seed: " << randseed << endl;

	gsl_rng_set(rng, randseed);
	int N = atoi(argv[1]);
	int archs = atoi(argv[2]);

	double mingamma = strtod(argv[3], NULL);
	double deltagamma = strtod(argv[4], NULL);
	int steps = atoi(argv[5]);

	int starts = 10000, nets = 20;

	int maxcyclen = 20;
	cout << "#starts: " << starts << "\n";
	cout << "#gamma\tC";
	for(int i = 1; i <= maxcyclen; i++)
		cout << "\tC_" << i;
	cout << "\n";


	for(int z = 0; z < steps; z++)
	{
		double gamma = mingamma + deltagamma * z;

		vector<double> output(maxcyclen + 1, 0);
		for(int arch = 0; arch < archs; arch++)
		{
			knet net;
			if(!powerlawNetwork(net, N, 1, gamma, gamma, true))
				return 1;
			Knet::prepareNetNoshrink(net);
//			cerr << gamma << ": " << arch << "/" << archs;
			Knet::gatherStatisticsOnCycles(net, starts, nets,
				cerr, true, maxcyclen, output);
		}

		double archsd = (double)archs;
		cout << -gamma;
		for(int i = 0; i <= maxcyclen; i++)
		{
			double cx = output[i] / archsd;
			cout << "\t" << cx;
		}
		cout << endl;

	}

	return 0;
}

