/**
 * Filename     : baseCompartmentNeighborhood.h
 * Description  : The common base for classes describing comp updates
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : September 2005
 * Revision     : $Id: baseCompartmentNeighborhood.h,v 1.3 2006/03/18 00:05:14 henrik Exp $
 */
#ifndef BASECOMPARTMENTNEIGHBORHOOD_H
#define BASECOMPARTMENTNEIGHBORHOOD_H

#include<vector>
#include<string>
#include<iostream>
#include<fstream>

//#include"organism.h"
#include"compartment.h"

class Organism;

//! A base class for describing a compartment neighborhood
/*! The BaseCompartmentNeighborhood class is a base class used to
  define different types of compartmental neighborhoods. Each class
  uses a vector of parameters and variable indeces to define and
  update which compartments are neighbors to each other.  The variable
  indeces are devided into multiple layers to allow for different
  types of contributions for different variables. The
  baseCompartmentNeighborhood class can be seen as a 'factory'
  creating different types of neighborhood definitions.
*/ 
class BaseCompartmentNeighborhood {
  
 private:
  
  std::string id_;
  std::vector<double> parameter_;           
  std::vector<std::string> parameterId_;           
  std::vector< std::vector<size_t> > variableIndex_;
  double previousTime_;
  unsigned int numNeighbor_;

 public:
  
  //Make these private...
  inline unsigned int numNeighbor() const;
  inline void setNumNeighbor(unsigned int value);  


  static BaseCompartmentNeighborhood* 
    createCompartmentNeighborhood(std::vector<double> &paraValue, 
				  std::vector< std::vector<size_t> > 
				  &indValue, std::string idValue );
  static BaseCompartmentNeighborhood* 
    createCompartmentNeighborhood( std::istream &IN ); 
  
  //Constructor/destructor not defined!
  //BaseCompartmentNeighborhood();
  virtual ~BaseCompartmentNeighborhood();
  
  BaseCompartmentNeighborhood & 
    operator=( const BaseCompartmentNeighborhood & 
	       baseCompartmentNeighborhoodCopy );
  
  // Get values
  inline std::string id() const;
  inline size_t numParameter() const;  
  inline size_t numVariableIndexLevel() const;
  inline size_t numVariableIndex(size_t level) const;
  
  inline double parameter(size_t i) const;
  inline double& parameterAddress(size_t i);
  inline std::string parameterId(size_t i) const;
  inline size_t variableIndex(size_t i,size_t j) const;
  inline double previousTime() const;
  
  // Set values
  inline void setId(std::string value);
  inline void setParameter(size_t i,double value);
  inline void setParameter(std::vector<double> &value);
  inline void setParameterId(size_t i,std::string value);
  inline void setParameterId(std::vector<std::string> &value);
  inline void setVariableIndex(size_t i, size_t j,size_t value);
  inline void setVariableIndex(size_t i, std::vector<size_t> &value);
  inline void setVariableIndex(std::vector< std::vector<size_t> > &value);
  inline void setPreviousTime(double value);  
  
  virtual unsigned int create(Organism &O,std::vector< std::vector<double> > &y,
			      double t=0.0 );
  virtual unsigned int update(Organism &O, 
			      std::vector< std::vector<double> > &y,
			      double t=0.0 );  
};

//!Returns the id string
inline std::string BaseCompartmentNeighborhood::id() const {
  return id_;
}

//!Returns the number of parameters
inline size_t BaseCompartmentNeighborhood::numParameter() const {
  return parameter_.size();
}

//!Returns the number of variable-index levels
inline size_t BaseCompartmentNeighborhood::numVariableIndexLevel() const {
  return variableIndex_.size();
}

//!Returns the number of variable-index within one level 
inline size_t BaseCompartmentNeighborhood::numVariableIndex(size_t level) const {
  return variableIndex_[level].size();
}

//!Returns a parameter
inline double BaseCompartmentNeighborhood::parameter(size_t i) const {
  return parameter_[i];
}

//!Returns a reference of a parameter
inline double& BaseCompartmentNeighborhood::parameterAddress(size_t i) {
  return parameter_[i];
}

//!Returns the id for a parameter
inline std::string BaseCompartmentNeighborhood::parameterId(size_t i) const {
  return parameterId_[i];
}

//!Returns the index of variable used in the update equation
inline size_t BaseCompartmentNeighborhood::variableIndex(size_t i,size_t j) const {
  return variableIndex_[i][j];
}

//!Returns the previous time for update
inline double BaseCompartmentNeighborhood::previousTime() const {
  return previousTime_;
}

//!Returns the current number of neighbors
inline unsigned int BaseCompartmentNeighborhood::
numNeighbor() const {
  return numNeighbor_;
}

//!Sets the id string
inline void BaseCompartmentNeighborhood::setId(std::string value) {
  id_=value;
}

//!Sets the parameter value with index i
inline void BaseCompartmentNeighborhood::setParameter(size_t i,double value) {
  parameter_[i]=value;
}

//!Sets all parameters within a compartmentNeighborhood
inline void BaseCompartmentNeighborhood::
setParameter(std::vector<double> &value) {
  parameter_ = value;
}

//!Sets the parameter id value with index i
inline void BaseCompartmentNeighborhood::
setParameterId(size_t i,std::string value) {
  parameterId_[i]=value;
}

//!Sets all parameter id's within a compartmentNeighborhood
inline void BaseCompartmentNeighborhood::
setParameterId(std::vector<std::string> &value) {
  parameterId_=value;
}

//!Sets the variable index at specific level and place
inline void BaseCompartmentNeighborhood::
setVariableIndex(size_t i, size_t j,size_t value) {
  variableIndex_[i][j] = value;
}

//!Sets the variable indeces for a given level
inline void BaseCompartmentNeighborhood::
setVariableIndex(size_t i, std::vector<size_t> &value) {
  variableIndex_[i] = value;
}

//!Sets the variable indeces used in a compartmentNeighborhood
inline void BaseCompartmentNeighborhood::
setVariableIndex(std::vector< std::vector<size_t> > &value) {
  variableIndex_ = value;
}

//!Sets the time when updated
inline void BaseCompartmentNeighborhood::setPreviousTime(double value) {
  previousTime_=value;
}

//!Sets the current number of neighbors
inline void BaseCompartmentNeighborhood::
setNumNeighbor(unsigned int value) {
  numNeighbor_=value;
}

#endif


