/**
 * Filename     : baseReaction.cc
 * Description  : A base class describing variable updates
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: baseReaction.cc,v 1.25 2006/03/18 00:05:14 henrik Exp $
 */
#include<vector>

#include"baseReaction.h"
#include"creation.h"
#include"degradation.h"
#include"grn.h"
//#include"growth.h"
//#include"mechanical.h"
#include"transport.h"
#include"massAction.h"

BaseReaction::~BaseReaction(){}

//!Factory creator, all creation should be mapped onto this one 
/*! Given the idValue a reaction of the defined type is returned
(using new Class).*/
BaseReaction *
BaseReaction::createReaction(std::vector<double> &paraValue,
			       std::vector< std::vector<size_t> > &indValue, 
			       std::string idValue ) {
  
  //Grn type of (creation) reactions
  //grn.h,grn.cc
  if( idValue=="grn" ) 
    return new Grn(paraValue,indValue);
  //Creation
  //creation.h,creation.cc
  else if(idValue=="creationZero")
    return new CreationZero(paraValue,indValue);
  else if(idValue=="creationOne")
    return new CreationOne(paraValue,indValue);
  //Degradation
  //degradation.h,degradation.cc
  else if(idValue=="degradationOne")
    return new DegradationOne(paraValue,indValue);
  else if(idValue=="degradationTwo")
    return new DegradationTwo(paraValue,indValue);
  //Mass action types of reactions
  //massAction.h,massAction.cc
  else if( idValue=="massActionEnzymatic" )
    return new MassActionEnzymatic(paraValue,indValue);
  //Transport
  //transport.h,transport.cc
  else if(idValue=="diffusionSimple")
    return new DiffusionSimple(paraValue,indValue);
//Default, if nothing found
  else {
    std::cerr << "\nBaseReaction::createReaction() WARNING: Reactiontype " 
	      << idValue << " not known, no reaction created.\n\7";
    exit(-1);
  }
}

//!This creator reads from an open file and then calls for the main creator
BaseReaction* 
BaseReaction::createReaction(std::istream &IN ) {
  
  std::string idVal;
  size_t pNum,levelNum;
  IN >> idVal;
  IN >> pNum;
  IN >> levelNum;
  std::vector<size_t> varIndexNum( levelNum );
  for( size_t i=0 ; i<levelNum ; i++ )
    IN >> varIndexNum[i];
  
  std::vector<double> pVal( pNum );
  for( size_t i=0 ; i<pNum ; i++ )
    IN >> pVal[i];
  
  std::vector< std::vector<size_t> > varIndexVal( levelNum );
  for( size_t i=0 ; i<levelNum ; i++ )
    varIndexVal[i].resize( varIndexNum[i] );
  
  for( size_t i=0 ; i<levelNum ; i++ )
    for( size_t j=0 ; j<varIndexNum[i] ; j++ )
      IN >> varIndexVal[i][j];
  
  return createReaction(pVal,varIndexVal,idVal);
}

void BaseReaction::derivs(Compartment &compartment,size_t species,
			  std::vector< std::vector<double> > &y,
			  std::vector< std::vector<double> > &dydt ) {
  std::cerr << "BaseReaction::derivs() should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}  

void BaseReaction::update( double h ) {
  std::cerr << "BaseReaction::update() should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}

void BaseReaction::print( std::ofstream &os ) {
  std::cerr << "BaseReaction::print(ofstream) should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}
