/**
 * Filename     : compartment.cc
 * Description  : A class describing a compartment with neighborhood
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: compartment.cc,v 1.9 2006/03/18 12:23:08 henrik Exp $
 */

#include"compartment.h"
#include"topology.h"

Compartment::Compartment() {

  setIndex(static_cast<size_t>(-1));
  setVolume( 1.0 );
}


Compartment::Compartment( const Compartment & compartmentCopy ) {
  
  setIndex( compartmentCopy.index() );
  setId( compartmentCopy.id() );
  setTopology( compartmentCopy.topology() );
  setVariable( compartmentCopy.variable() );
  setNeighbor( compartmentCopy.neighbor() );
  setNeighborArea( compartmentCopy.neighborArea() );
  setVolume( compartmentCopy.volume() );
  setParameter( compartmentCopy.parameter() );
}

Compartment::Compartment( const Topology* topologyVal, 
			  size_t numSpeciesVal,size_t indexVal, 
			  std::string idVal) {
  setIndex( indexVal );
  setId( idVal );
  setTopology( topologyVal );
  
  setVolume( 1.0 );
  std::vector<double> tmpVarVal( numTopologyVariable()+numSpeciesVal );
  setVariable( tmpVarVal );
}

Compartment::Compartment( const Topology* topologyVal,
			  std::vector<double> &variableVal,
			  size_t indexVal, 
			  std::string idVal ) {
  setIndex( indexVal );
  setId( idVal );
  setTopology( topologyVal );
  setVariable( variableVal );
  
  std::vector<double> tmpD;
  std::vector<size_t> tmpI;
  setParameter(tmpD);
  setNeighbor(tmpI);
  setNeighborArea(tmpD);
  
}

Compartment::~Compartment() {
  //std::cerr << "Compartment::~Compartment()\n";
}

//!Returns the dimension of the compartment extracted from the embedding
size_t Compartment::numDimension() const {
  return topology_->numDimension();
}

//!Returns the number of topology variables extracted from the topology
size_t Compartment::numTopologyVariable() const {
  return topology_->numVariable();
}

//!Removes neighbor element with index i (returns the number of removed elem.)
unsigned int Compartment::removeNeighbor(size_t index) {
  
  if( numNeighborLevel()<1 ) return 0;
  unsigned int removed=0;
  std::vector<size_t> newNeigh;
  for( size_t n=0 ; n<numNeighbor() ; n++ ) {
    if( neighbor(n)==index )
      removed++;
    else
      newNeigh.push_back( neighbor(n) );
  }
  setNeighbor(newNeigh);
  return removed;
}

//!Changes index in the neighbor vector from old to new and returns the number
unsigned int Compartment::changeNeighborIndex(size_t oldIndex,
					      size_t newIndex) {
  
  if( numNeighborLevel()<1 ) return 0;
  unsigned int moved=0;
  for( size_t n=0 ; n<numNeighbor() ; n++ ) {
    if( neighbor(n)==oldIndex ) {
      setNeighbor(n,newIndex);
      moved++;
    }
  }
  return moved;
}

//!Removes neighbor element with index i at level l
unsigned int Compartment::removeNeighborAtLevel(size_t l,size_t index) {
  
  if( numNeighborLevel()<l+1 ) return 0;
  unsigned int removed=0;
  std::vector<size_t> newNeigh;
  for( size_t n=0 ; n<numNeighborAtLevel(l) ; n++ ) {
    if( neighborAtLevel(l,n)==index )
      removed++;
    else
      newNeigh.push_back( neighborAtLevel(l,n) );
  }
  setNeighborAtLevel(l,newNeigh);
  return removed;
}

//!Changes index in the neighbor vector from old to new at level l
unsigned int Compartment::
changeNeighborIndexAtLevel(size_t l,size_t oldIndex,size_t newIndex) {

  if( numNeighborLevel()<l+1 ) return 0;
  unsigned int moved=0;
  for( size_t n=0 ; n<numNeighborAtLevel(l) ; n++ ) {
    if( neighborAtLevel(l,n)==oldIndex ) {
      setNeighborAtLevel(l,n,newIndex);
      moved++;
    }
  }
  return moved;
}

//!Sets the topology pointer
void Compartment::setTopology(const Topology* value) {
  topology_ = value;
}

//!Updates the neighbor parameters according to Mitchisons rules
/*! This is a preliminary function to be able to test Mitchisons flux model
 *  from Mitchison (1981)*/
// void Compartment::updateParametersMitchison( std::vector< 
// 					     std::vector<double> > &y,
// 					     Compartment &cN, double h) {
  
//   //Define update parameters
//   double epsM = 0.0001;
//   double kM=0.01;
//   double qM=0.06;
//   double truncVal=1.0;//Maximal value for a p (to avoid divergencies)
  
//   //Check that parameters are defined
//   if( numParameter()<numNeighbor() ) {
//     std::cerr << "Compartment::updateParametersMitchison "
// 	      << "Wrong number of compartment parameters.\n";
//     exit(-1);
//   }
//   int i = index();
//   int j = cN.index();
  
//   //Find positions in neighbor vectors
//   int k=-1,kN=-1,count=0;
//   for( int kTmp=0 ; kTmp<numNeighbor() ; kTmp++ ) 
//     if( neighbor(kTmp)==j ) {
//       k=kTmp;
//       count++;
//     }
//   if( count != 1 ) {
//     std::cerr << "Compartment::updateParametersMitchison "
// 	      << "Wrong neighborhood to i.\n";
//     exit(-1);
//   }
//   count=0;
//   for( int kTmp=0 ; kTmp<cN.numNeighbor() ; kTmp++ ) 
//     if( cN.neighbor(kTmp)==i ) {
//       kN=kTmp;
//       count++;
//     }
//   if( count != 1 ) {
//     std::cerr << "Compartment::updateParametersMitchison "
// 	      << "Wrong neighborhood to j.\n";
//     exit(-1);
//   }
  
//   //Calculate flux (assuming flux molecule as first species)
//   int yCol=speciesStart();
//   double F=parameter(k)*y[i][yCol]-cN.parameter(kN)*y[j][yCol];
  
//   //Update parameters according to Mitchison
//   double truncMax = (kM*F*F>truncVal) ? truncVal : kM*F*F;
//   //std::cerr << truncMax << " " << kM*F*F << " " << truncVal << "\n"; 
//   if( F>0.0 ) {
//     double p1New=parameter(k)+h*epsM*(truncMax+qM-parameter(k));
//     if( p1New<0.0 ) {
//       p1New=0.0;
//       std::cerr << "Compartment::updateParametersMitchison "
// 		<< "(F>0) Negative p1 adjusted to 0.\n";
//     }
//     setParameter(k,p1New);
//     double p2New=cN.parameter(kN)+h*epsM*(qM-cN.parameter(kN));
//     if( p2New<0.0 ) {
//       p2New=0.0;
//       std::cerr << "Compartment::updateParametersMitchison "
// 		<< "(F>0) Negative p2 adjusted to 0.\n";
//     }
//     cN.setParameter(kN,p2New);
//   }
//   else if( F<0.0 ) {
//     double p1New=parameter(k)+h*epsM*(qM-parameter(k));
//     if( p1New<0.0 ) {
//       p1New=0.0;
//       std::cerr << "Compartment::updateParametersMitchison "
// 		<< "(F<0) Negative p1 adjusted to 0.\n";
//     }
//     setParameter(k,p1New);
//     double p2New=cN.parameter(kN)+h*epsM*(truncMax+qM-cN.parameter(kN));
//     if( p2New<0.0 ) {
//       std::cerr << "Compartment::updateParametersMitchison "
// 		<< "(F<0) Negative p2 adjusted to 0.\n";
//       p2New=0.0;
//     }
//     cN.setParameter(kN,p2New);
//   }
//   //else No update...
// }

