/**
 * Filename     : compartment.h
 * Description  : A class describing a compartment with neighborhood
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: compartment.h,v 1.8 2006/03/18 00:05:14 henrik Exp $
 */
#ifndef COMPARTMENT_H
#define COMPARTMENT_H

#include<vector>
#include<string>
#include<iostream>
#include<fstream>

class Topology;

//! A class describing properties of a compartment sa embedding and topology
/*!The Compartment handles the update and information of the embedding
  and topology. It holds a vector of variable values for all dynamical
  variables in the compartment. It holds the neighbor information, ie
  the neighbor indeces and if applicable the cross section areas. It
  can also store a vector of parameters that can be updated during a
  simulation, and used in baseReaction calculations of the
  derivatives.*/ 
class Compartment {
  
 private:
  
  size_t index_;
  std::string id_;           
  std::vector<double> variable_;
  std::vector< std::vector<size_t> > neighbor_;
  std::vector<double> neighborArea_;
  double volume_;
  std::vector<double> parameter_;
  const Topology* topology_;
  
 public:
  
  Compartment();
  Compartment( const Compartment & compartmentCopy );
  Compartment( const Topology* topologyVal, size_t numSpecies, 
	       size_t index=static_cast<size_t>(-1), 
	       std::string idValue="");
  Compartment( const Topology* topologyVal, 
	       std::vector<double> &variableVal,
	       size_t index=static_cast<size_t>(-1), 
	       std::string idValue="" );
  //Compartment( std::istream &IN );
  //Compartment( char *inFile );
  //Compartment( std::string inFile );
  
  ~Compartment();
  
  //Compartment & operator=( const Compartment & compartmentCopy );
  
  // Get values
  inline std::string id() const;
  inline size_t index() const;
  
  inline const Topology* topology() const;
  
  inline size_t numVariable() const;  
  inline double variable(size_t i) const;
  inline const std::vector<double> variable() const;

  inline size_t numNeighbor() const;
  inline size_t neighbor(size_t i) const;
  inline const std::vector<size_t> neighbor() const;
  inline size_t numNeighborLevel() const;
  inline size_t numNeighborAtLevel(size_t l) const;
  inline size_t neighborAtLevel(size_t l,size_t i) const;
  inline const std::vector<size_t> neighborAtLevel(size_t l) const;
  
  inline size_t numNeighborArea() const;
  inline double neighborArea(size_t i) const;
  inline const std::vector<double> neighborArea() const;
  
  inline double volume() const;

  inline size_t topologyStart() const;
  size_t numTopologyVariable() const;
  size_t numDimension() const;
  inline size_t speciesStart() const;
  inline size_t numSpecies() const;

  inline size_t numParameter() const;  
  inline double parameter(size_t i) const;
  inline const std::vector<double> parameter() const;
  
  
  // Set values
  inline void setId(std::string value);
  inline void setIndex(size_t value);
  void setTopology(const Topology* value);
  inline void setVariable(size_t i,double value);
  inline void setVariable(const std::vector<double> &value);
  inline void setNeighbor(size_t i,size_t value);
  inline void setNeighbor(const std::vector<size_t> &value);
  inline void addNeighbor(size_t value);
  unsigned int removeNeighbor(size_t index);
  unsigned int changeNeighborIndex(size_t oldIndex,size_t newIndex);
  inline void setNeighborAtLevel(size_t l,size_t i,size_t value);
  inline void setNeighborAtLevel(size_t l,const std::vector<size_t> &value);
  inline void addNeighborAtLevel(size_t l,size_t value);
  unsigned int removeNeighborAtLevel(size_t l,size_t index);
  unsigned int changeNeighborIndexAtLevel(size_t l,size_t oldIndex,size_t newIndex);

  inline void setNeighborArea(size_t i,double value);
  inline void setNeighborArea(const std::vector<double> &value);
  inline void addNeighborArea(double value);
  inline void setVolume(double value);
  inline void setParameter(size_t i,double value);
  inline void setParameter(const std::vector<double> &value);
  
//   void updateParametersMitchison( std::vector< std::vector<double> > &y,
// 				  Compartment &cN, double h);
  
};

//!Returns the id of the compartment
inline std::string Compartment::id() const {
  return id_;
}

//!Returns the index of the compartment
inline size_t Compartment::index() const {
  return index_;
}

//!Returns the number of species
inline size_t Compartment::numVariable() const {
  return variable_.size();
}

//!Returns the number of neighbors
inline size_t Compartment::numNeighbor() const {
  return neighbor_.size()>0 ? neighbor_[0].size() : 0;
}

//!Returns the number of neighbor levels (one if only nearest neighbors)
inline size_t Compartment::numNeighborLevel() const {
  return neighbor_.size();
}

//!Returns the number of neighbor at level l
inline size_t Compartment::numNeighborAtLevel(size_t l) const {
  return neighbor_.size()>l ? neighbor_[l].size() : 0;
}

//!Returns the number of neighbor areas (should be the same as number of neigh)
inline size_t Compartment::numNeighborArea() const {
  return neighborArea_.size();
}

//!Returns the volume
inline double Compartment::volume() const {
  return volume_;
}

//!Returns the topology pointer
inline const Topology* Compartment::topology() const {
  return topology_;
}

//!Returns a variable value
inline double Compartment::variable(size_t i) const {
  return variable_[i];
}

//!Returns the vector of variables
inline const std::vector<double> Compartment::variable() const{
  return variable_;
}

//!Returns a neighbor
inline size_t Compartment::neighbor(size_t i) const {
  if( neighbor_.size()<1 ) {
    std::cerr << "Comparment::neighbor(i) No neighs defined.\n";
    exit(-1);
  }
  return neighbor_[0][i];
}

//!Returns the vector of neighbors
inline const std::vector<size_t> Compartment::neighbor() const{
  if( neighbor_.size()<1 ) {
    std::cerr << "vector<int> Comparment::neighbor(i) "
	      << "No neighs defined.\n";
    exit(-1);
  }
  return neighbor_[0];
}

//!Returns a neighbor at level l
inline size_t Compartment::neighborAtLevel(size_t l,size_t i) const {
  if( neighbor_.size()<l ) {
    std::cerr << "int Comparment::neighbor(l,i) "
	      << "No neighs defined at level " << l << ".\n";
    exit(-1);
  }
  return neighbor_[l][i];
}

//!Returns the vector of neighbors at level l
inline const std::vector<size_t> Compartment::neighborAtLevel(size_t l) const{
  if( neighbor_.size()<l ) {
    std::cerr << "vector<int> Comparment::neighbor(l,i) "
	      << "No neighs defined at level " << l << ".\n";
    exit(-1);
  }
  return neighbor_[l];
}

//!Returns a neighbor area
inline double Compartment::neighborArea(size_t i) const {
  return neighborArea_[i];
}

//!Returns the vector of neighborAreas
inline const std::vector<double> Compartment::neighborArea() const{
  return neighborArea_;
}

//!Returns the start position of the topology in the variable vector 
/*! The topologyStart is always equal to zero since the variables are
  organized as topology,species.*/ 
inline size_t Compartment::topologyStart() const {
  return 0;
}

//!Returns the start position of the species in the variable vector 
/*! The start position of the species within a compartment should
  always be the number of topology variables since
  the variables are organized as topology,species. */
inline size_t Compartment::speciesStart() const {
  return numTopologyVariable();
}

//!Returns the number of species variables calculated as numVar-(dim+numTopol)
inline size_t Compartment::numSpecies() const {
  return numVariable()-numTopologyVariable();
}

//!Returns the number of parameters
inline size_t Compartment::numParameter() const {
  return parameter_.size();
}

//!Returns a parameter value
inline double Compartment::parameter(size_t i) const {
  return parameter_[i];
}

//!Returns the vector of parameters
inline const std::vector<double> Compartment::parameter() const{
  return parameter_;
}

//!Sets the id of the compartment
inline void Compartment::setId(const std::string value) {
  id_ = value;
}

//!Sets the index of the compartment
inline void Compartment::setIndex( size_t value ) {
  index_ = value;
}

//!Sets the speciesConcentrations from a vector
inline void 
Compartment::setVariable( const std::vector<double> &value) {
  variable_ = value;
}

//!Sets a speciesConcentration value
inline void Compartment::setVariable(size_t i, double value) {
  variable_[i] = value;
}

//!Sets the neighbors from a vector
inline void Compartment::setNeighbor( const std::vector<size_t> &value) {
  if( neighbor_.size()<1 )
    neighbor_.resize(1);
  neighbor_[0] = value;
}

//!Sets a neighbor
inline void Compartment::setNeighbor(size_t i, size_t value) {
  if( neighbor_.size()<1 )
    neighbor_.resize(1);
  if( neighbor_[0].size()<=i )
    neighbor_[0].resize(i+1);
  neighbor_[0][i] = value;
}

//!Adds a neighbor
inline void Compartment::addNeighbor(size_t value) {
  if( neighbor_.size()<1 )
    neighbor_.resize(1);
  neighbor_[0].push_back(value);
}

//!Sets the neighbors at level l from a vector
inline void Compartment::
setNeighborAtLevel(size_t l, const std::vector<size_t> &value) {
  if( neighbor_.size()<=l )
    neighbor_.resize(l+1);
  neighbor_[l] = value;
}

//!Sets a neighbor at level l
inline void Compartment::setNeighborAtLevel(size_t l,size_t i, 
					    size_t value) {
  if( neighbor_.size()<=l )
    neighbor_.resize(l+1);
  if( neighbor_[0].size()<=i )
    neighbor_[0].resize(i+1);
  neighbor_[l][i] = value;
}

//!Adds a neighbor at level l
inline void Compartment::addNeighborAtLevel(size_t l,size_t value) {
  if( neighbor_.size()<=l )
    neighbor_.resize(l+1);
  neighbor_[l].push_back(value);
}

//!Sets the neighbor areas from a vector
inline void Compartment::setNeighborArea( const std::vector<double> &value) {
  neighborArea_ = value;
}

//!Sets a neighbor area
inline void Compartment::setNeighborArea(size_t i, double value) {
  neighborArea_[i] = value;
}

//!Adds a neighbor area
inline void Compartment::addNeighborArea(double value) {
  neighborArea_.push_back(value);
}

//!Sets the volume
inline void Compartment::setVolume(double value) {
  volume_ = value;
}

//!Sets the parameters from a vector
inline void 
Compartment::setParameter( const std::vector<double> &value) {
  parameter_ = value;
}

//!Sets a parameter value
inline void Compartment::setParameter(size_t i, double value) {
  parameter_[i] = value;
}

#endif
