/**
 * Filename     : degradation.cc
 * Description  : Classes describing degradation updates
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: degradation.cc,v 1.4 2006/03/18 12:23:08 henrik Exp $
 */
#include"degradation.h"
#include"baseReaction.h"

//!Constructor for the DegradationOne class
DegradationOne::DegradationOne(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > 
			       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "DegradationOne::DegradationOne() "
	      << "Uses only one parameter k_deg (lambda)\n";
    exit(0);
  }
  if( indValue.size() ) {
    std::cerr << "DegradationOne::DegradationOne() "
	      << "No variable index are used.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("degradationOne");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_deg";
  setParameterId( tmp );
}

//! Derivative contribution for a degradation proportional to concentration
/*! Deriving the time derivative contribution from a degradation
  using the values in y and add the results to dydt. Compartment is
  the cell and varIndex is the index of the variable to be
  updated. The parameter is k_deg in dc/dt -= k_deg * c
*/
void DegradationOne::derivs(Compartment &compartment,size_t varIndex,
			    std::vector< std::vector<double> > &y,
			    std::vector< std::vector<double> > &dydt ) {
  
  dydt[compartment.index()][varIndex] -= parameter(0)*
    y[compartment.index()][varIndex];
}

//!Constructor for the DegradationTwo class
DegradationTwo::DegradationTwo(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > 
			       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "DegradationTwo::DegradationTwo() "
	      << "Uses only one parameter k_deg\n";
    exit(0);
  }
  if( indValue.size() != 1 || indValue[0].size() != 1 ) {
    std::cerr << "DegradationTwo::DegradationTwo() "
	      << "One variable index is used.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("degradationTwo");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_deg";
  setParameterId( tmp );
}

//! Derivative contribution for a degradation proportional to concentrations
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and varIndex is the index of the variable to be updated. The
  parameter is k_deg in dc/dt = k_deg * c *c_A , where c_A is the
  concentration of another species.  */
void DegradationTwo::derivs(Compartment &compartment,size_t varIndex,
			    std::vector< std::vector<double> > &y,
			    std::vector< std::vector<double> > &dydt ) {
  
  dydt[compartment.index()][varIndex] -= parameter(0)*
    y[compartment.index()][varIndex]*
    y[compartment.index()][variableIndex(0,0)];
}
