/**
 * Filename     : grn.cc
 * Description  : Classes describing gene regulatory updates
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: grn.cc,v 1.3 2006/03/18 00:05:14 henrik Exp $
 */
#include"grn.h"
#include"baseReaction.h"

//!Constructor for the Grn reaction class
Grn::Grn(std::vector<double> &paraValue, 
	 std::vector< std::vector<size_t> > 
	 &indValue ) {

  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()<2 ) {
    std::cerr << "Grn::Grn() At least two parameters (h,tau) must be given.\n";
    exit(0);
  }
  if( indValue.size()>1 ) {
    std::cerr << "Grn::Grn() "
	      << "At most one level of variable index can be used.\n";
    exit(0);
  }
  if( ( indValue.size() && paraValue.size() != indValue[0].size()+2 ) ||
      ( indValue.size()==0 && paraValue.size() != 2 ) ) {
    std::cerr << "Grn::Grn() "
	      << "Wrong number of parameters/variable indeces given.\n";

    std::cerr << paraValue.size() << " " << indValue.size();
    if( indValue.size() )
      std::cerr << " " << indValue[0].size();
    std::cerr << "\n";
    exit(0);
  }
  
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("grn");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "h";
  tmp[1] = "tau";
  for( size_t i=2 ; i<numParameter() ; i++ )
    tmp[i] = "T";
  setParameterId( tmp );
}

//! Derivative contribution for one grn protein
/*! Deriving the time derivative contribution from grn for a protein
  using the values in y and add the results to dydt. compartment is
  the number of the cell and specie is the index of the variable to be
  updated.
  Parameters are situated as h,tau,T...
*/
void Grn::derivs(Compartment &compartment,size_t species,
		 std::vector< std::vector<double> > &y,
		 std::vector< std::vector<double> > &dydt ) {
  double u;
  
  //Threshold
  u = parameter(0);//h
  
  // Internal contribution
  size_t add=2;
  if( numVariableIndexLevel() )
    for(size_t b=0 ; b<numVariableIndex(0) ; b++ )
      u += parameter(b+add)*y[compartment.index()][variableIndex(0,b)];
  
  // Apply sigmoid and tau parameter
  if( parameter(1)>0. )
    dydt[compartment.index()][species] += sigmoid(u)/parameter(1);
  else {
    std::cerr << "Grn::derivs Division by tau=0.\n";
    exit(-1);
  }
}

