/**
 * Filename     : organism.cc
 * Description  : A class describing an organism 
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: organism.cc,v 1.21 2006/03/18 12:23:08 henrik Exp $
 */

#include<cmath>
#include"organism.h"

Organism::Organism() {

  setNumTopology(0);
  setNumNeighborhood(0);
}

Organism::Organism( const Organism & organismCopy ) {
  
  std::cerr << "Organism::Organism(const Organism &)"
	    << " Not implemented yet...\n";
  exit(0);
}

Organism::Organism( Topology* topologyVal, 
		    std::vector<Species> &speciesVal,std::string idValue,
		    std::string initFile) {

  std::cerr << "Organism::Organism(topol,speciesval,idval,initFile)"
	    << " Not implemented yet...\n";
  exit(0);
}

Organism::Organism( const char *modelFile, const char *initFile, 
		    int verbose ) {

  readOrganism(modelFile,initFile,verbose);
}

Organism::Organism( std::string modelFile, std::string initFile, 
		    int verbose ) {
  
  readOrganism(modelFile,initFile,verbose);
}

Organism::~Organism() {
  //std::cerr << "Organism::~Organism()\n";
}

//!Reads a model and init file into the organism
void Organism::readOrganism( const char *modelFile, const char *initFile, 
			     int verbose ) {
  
  //Read topology, species, and reaction models from file
  if( verbose )
    std::cerr << "Organism::readOrganism(char*,char*) reading model file " 
	      << modelFile << "\n";
  readModel(modelFile);
  if( verbose )
    std::cerr << "Model File Read!\n";
  
  //Read initial values from file and create compartments
  if( std::string(initFile) != "") {
    if( verbose )
      std::cerr << "Organism::readOrganism(char*,char*) reading init file " 
		<< initFile << ".\n";
    readInit(initFile);
    if( verbose )
      std::cerr << "Init file read!\n";
  }
  else {
    std::cerr << "Organism::readOrganism( char*, char* ) "
	      << "Warning! No initial configuration read.\n";
  }
}

//!Reads a model file and init file into the organism (by using char version) 
void Organism::readOrganism( std::string modelFile, std::string initFile, 
			int verbose ) {
  if( verbose )
    std::cerr << "Organism::readOrganism( std::string,std::string,int)\n";
  const char* mFile=modelFile.c_str();
  const char* iFile=initFile.c_str();
  readOrganism(mFile,iFile,verbose);
}

//!Reads a model from an open file
void Organism::readModel(std::ifstream &IN) {
  
  unsigned int numTopologyVal,numSpeciesVal,numReactionVal,
    numNeighborhoodVal;
  std::string idVal;
  IN >> idVal;
  IN >> numTopologyVal;
  IN >> numSpeciesVal;
  IN >> numReactionVal;
  IN >> numNeighborhoodVal;
  
  if( numTopologyVal != 0 && numTopologyVal != 1 ) {
    std::cerr << "Organism::readModel(ifstream) - "
	      << "Zero or one topology is allopwed for right now! " 
	      << numTopologyVal << ".\n\n\7";exit(-1);}
  if( numNeighborhoodVal != 0 && numNeighborhoodVal != 1 ) {
    std::cerr << "Organism::readModel(ifstream) - "
	      << "Zero or one neighborhood is allowed for right now! " 
	      << numNeighborhoodVal << ".\n\n\7";exit(-1);}
  setId( idVal );
  setNumTopology( numTopologyVal );
  setNumNeighborhood( numNeighborhoodVal );
  
  //Read topology
  //////////////////////////////////////////////////////////////////////
  if( numTopology() ) {
    std::cerr << "org:readModel topology_.readTopology(IN)\n";
    topology_.readTopology(IN);
    std::cerr << "Topology " << topology().id() << " added with "
	      << topology().numVariable() << " variables, " 
	      << topology().numReaction() << " reactions and "
	      << topology().numCompartmentChange() 
	      << " compartment changes\n";
  }
  //Read Species
  //////////////////////////////////////////////////////////////////////
  //Remove any present species before adding
  if( numSpecies() ) 
    species_.resize(0);
  
  for( unsigned int i=0 ; i<numSpeciesVal ; i++ ) {
    addSpecies(IN);  
    std::cerr << "Species " << species(i).id() << " added with index "
	      << species(i).index() << " and " << species(i).numReaction()
	      << " reactions.\n"; 
  }
  
  //Read Reactions
  //////////////////////////////////////////////////////////////////////
  //Remove any present reactions before adding
  if( numReaction() ) 
    reaction_.resize(0);
  
  for( unsigned int i=0 ; i<numReactionVal ; i++ )
    if( addReaction(IN) )
      std::cerr << "Organism::ReadModel(ifstream) "
		<< "Warning Adding reaction failed for "
		<< "organism " << id() << " (index " << i << ")\n";
  
 
  //Read Neighborhood
  //////////////////////////////////////////////////////////////////////
  if( numNeighborhood() )
    if( addNeighborhood(IN) )
      std::cerr << "Organism::ReadModel(ifstream) "
		<< "Warning Adding neighborhood failed for "
		<< "organism " << id() << ".\n";
}

//!Reads a model from a file
void Organism::readModel(const char *fileName) {

  std::ifstream IN(fileName);
  if( !IN ) {
    std::cerr << "Organism::readModel(char*) - "
	      << "Cannot open file " << fileName << "\n\n\7";exit(-1);}
  readModel(IN);
}

//!Reads init values from file 
void Organism::readModel(std::string fileName) {
  
  const char* fName = fileName.c_str();
  std::ifstream IN(fName);
  if( !IN ) {
    std::cerr << "Organism::readModel(std::string) - "
	      << "Cannot open file " << fileName << "\n\n\7";exit(-1);}
  readModel(IN);
}

//!Read an initial configuration from an open file
void Organism::readInit(std::ifstream &IN) {
  
  //Remove all compartments before adding
  if( numCompartment() )
    compartment_.resize(0);
  
  size_t numCompartmentVal,numVariableVal;
  IN >> numCompartmentVal;
  IN >> numVariableVal;
  if( numVariableVal != numTopologyVariable()+numSpecies() ) {
    std::cerr << "Organism::readInit(std::ifstream ) "
	      << "Wrong number of variables in init file.\n";
    std::cerr << numVariableVal << " in init file and " 
	      << numTopologyVariable() <<  "+" << numSpecies() 
	      << " in organsim.\n";
    exit(-1);
  }
  std::vector<double> varTmp( numVariableVal );
  for( size_t i=0 ; i<numCompartmentVal ; i++ ) {
    for( size_t j=0 ; j<numVariableVal ; j++ ) {
      IN >> varTmp[j];
    }
    Compartment tmpCompartment(topologyPointer(), varTmp, i, "");
    addCompartment( tmpCompartment );
  }
}

//!Reads init values from file 
void Organism::readInit(const char *fileName) {
  
  std::ifstream IN(fileName);
  if( !IN ) {
    std::cerr << "Organism::readInit(char*) - "
	      << "Cannot open file " << fileName << "\n\n\7";exit(-1);}
  readInit(IN);
}

//!Reads init values from file 
void Organism::readInit(std::string fileName) {
  
  const char* fName = fileName.c_str();
  std::ifstream IN(fName);
  if( !IN ) {
    std::cerr << "Organism::readInit(std::string) - "
	      << "Cannot open file " << fileName << "\n\n\7";exit(-1);}
  readInit(IN);
}

//!Adds a reaction to the list from an open file
int Organism::addReaction( std::istream &IN ) {
  if( !IN ) return -1;
  reaction_.push_back( BaseReaction::createReaction(IN) );
  return 0;
}

//!Initiates the parameter vector for the complete organism
/*! This function collects all parameters in all species>reactions and
  reactions into a single vector containing all the parameters for the
  complete organism (model).*/
void Organism::initiateParameter() {
  
  parameter_.resize(0);

  //collect topology parameters
  for( size_t i=0 ; i<topology_.numReaction() ; i++  )
    for( size_t j=0 ; j<topology_.reaction(i)->numParameter() ; j++  )
      parameter_.push_back( &( topology_.reaction(i)->parameterAddress(j) ) );
  
  //collect species-reaction parameters
  for( size_t k=0 ; k<numSpecies() ; k++  )
    for( size_t i=0 ; i<species(k).numReaction() ; i++  )
      for( size_t j=0 ; j<species(k).reaction(i)->numParameter() ; j++  )
	parameter_.push_back( &( species(k).reaction(i)->parameterAddress(j) ));
  
  //collect reaction parameters
  for( size_t i=0 ; i<numReaction() ; i++  )
    for( size_t j=0 ; j<reaction(i)->numParameter() ; j++  )
      parameter_.push_back( &( reaction(i)->parameterAddress(j) ) );  
}

//!Creates the neighborhood according to the defined neighborhood rule
/*! In the member class neighborhood_ a rule for how to extract and
  update the neighborhood for all compartments are defined.*/
void Organism::neighborhoodCreate(std::vector<std::vector<double> > &y,
				  double t ) { 
  if( numNeighborhood() )
    neighborhoodPointer()->create((*this),y,t); 
}

//!Updates the neighborhood according to the defined neighborhood rule
void Organism::neighborhoodUpdate(std::vector< std::vector<double> > &y,
				  double t ) {
  if( numNeighborhood() )
    neighborhoodPointer()->update((*this),y,t);
}

// //! Find (potential) neighbors to spherical compartments
// /*! Simple O(N^2) function for finding neighboring compartments.
//   rFactor gives a factor of the sum of the two cell radiuses for which
//   a threshold is set. If the distance is larger than the threshold,
//   the cells are not considered as neighbors. It returns the total
//   number of neighbors found.
// */
// int Organism::findNeighbors( std::vector< std::vector<double> > &y, 
// 			     double rFactor ) {

//   std::vector< std::vector<int> > neighbor(numCompartment());
//   double d,r;
//   int numNeigh=0;
  
//   int rIndex = dimension();
//   int eStart = 0;
  
//   for(int i=0 ; i<numCompartment() ; i++ )
//     for(int j=i+1 ; j<numCompartment() ; j++ ) {
//       r = y[i][rIndex] + y[j][rIndex];
//       d=0.;
//       for( int dim=eStart ; dim<rIndex ; dim++ )
// 	d += (y[i][dim]-y[j][dim])*(y[i][dim]-y[j][dim]);
//       d = sqrt( d );
      
//       if( d<= r*rFactor ) {
//         //Add neighbor
//         neighbor[i].push_back(j);
//         neighbor[j].push_back(i);
//         numNeigh++;
//       }
//     }
//   //Copy the new neighbours into the cells (Caveat: Not a good solution)
//   for(int i=0 ; i<numCompartment() ; i++ )
//     compartment(i).setNeighbor( neighbor[i] );
  
//   return numNeigh;
// }

// //! Find (potential) neighbors to elliptical compartments
// /*! Simple O(N^2) function for finding potential neighboring
//   elliptical compartments.  rFactor gives a factor of the sum of the
//   two cell radii for which a threshold is set. If the distance is
//   larger than the threshold, the cells are not considered as potential
//   neighbors. In this case of elliptical compartments, the radius is
//   replaced with a, the distance from the center to the ellipse
//   following the major axis. It returns the total number of neighbors
//   found.
// */
// int Organism::findNeighborsEllipse( std::vector< std::vector<double> > &y, 
// 				    double rFactor ) {

//   if( numTopology() != 5 ) {
//     std::cerr << "Organism::findNeighborsEllipse() "
// 	      << "Wrong number of topology variables.\n";
//     exit(-1);
//   }
//   //List columns used
//   int xaCol=0,yaCol=1,xbCol=2,ybCol=3;
//   //Define additional center points and variables
//   int dim=2;
//   double d,r,ai,aj;
//   int numNeigh=0;
//   std::vector< std::vector<int> > neighbor(numCompartment());
//   std::vector<double> ci(dim),cj(dim);
  
//   for(int i=0 ; i<numCompartment() ; i++ ) {
//     ci[0] = 0.5*( y[i][xaCol]+y[i][xbCol] );
//     ci[1] = 0.5*( y[i][yaCol]+y[i][ybCol] );
//     ai = 0.5*std::sqrt( (y[i][xaCol]-y[i][xbCol])*
// 			(y[i][xaCol]-y[i][xbCol])+
// 			(y[i][yaCol]-y[i][ybCol])*
// 			(y[i][yaCol]-y[i][ybCol]) );
//     for(int j=i+1 ; j<numCompartment() ; j++ ) {
//       cj[0] = 0.5*( y[j][xaCol]+y[j][xbCol] );
//       cj[1] = 0.5*( y[j][yaCol]+y[j][ybCol] );
//       aj = 0.5*std::sqrt( (y[j][xaCol]-y[j][xbCol])*
// 			  (y[j][xaCol]-y[j][xbCol])+
// 			  (y[j][yaCol]-y[j][ybCol])*
// 			  (y[j][yaCol]-y[j][ybCol]) );
//       r = ai+aj;
//       d = std::sqrt( (ci[0]-cj[0])*(ci[0]-cj[0])+ 
// 		     (ci[1]-cj[1])*(ci[1]-cj[1]) );
      
//       if( d<=r*rFactor ) {
//         //Add neighbor
//         neighbor[i].push_back(j);
//         neighbor[j].push_back(i);
//         numNeigh++;
//       }
//     }
//   }
//   //Copy the new neighbours into the cells (Caveat: Not a good solution)
//   for(int i=0 ; i<numCompartment() ; i++ )
//     compartment(i).setNeighbor( neighbor[i] );
  
//   return numNeigh;
// }

// //! Calculates neighbor overlaps given the radii and neighbors (2D)
// /*! Uses neighbor and size information to calculate the overlap
//   between spherical cells and calculates the intersection 'area'
//   between the neighbors. Finally stores the information in
//   compartment.neighboreSize.
//  */
// void Organism::findNeighborAreas( std::vector< std::vector<double> > &y ) {

//   int xIndex=0,yIndex=1;
//   int rIndex = dimension();
//   if( rIndex != 2 ) {
//     std::cerr << "Organism::findNeighborAreas: Only applicable to two "
// 	      << "dimensional systems.\n";
//     exit(-1);
//   }
//   for( int i=0 ; i<numCompartment() ; i++ ) {
    
//     int numNeigh = compartment(i).numNeighbor();
//     double X,Y,R;
//     int numBound=4;//Put the cell within a bounding square
//     int numLine = numNeigh+numBound;
//     std::vector<double> xk(numNeigh),yk(numNeigh),r(numNeigh),
//       dx(numLine),dy(numLine),xM(numLine),yM(numLine);
//     //Extract data from the y matrix of potential neighbors
//     //////////////////////////////////////////////////////////////////////
//     X = y[i][xIndex];
//     Y = y[i][yIndex];
//     R = y[i][rIndex];
//     for( int k=0 ; k<numNeigh ; k++ ) {
//       int j = compartment(i).neighbor(k);
//       xk[k] = y[j][xIndex];
//       yk[k] = y[j][yIndex];
//       r[k] = y[j][rIndex];
//     }
//     //calculate dx, dy, xM, yM for potential neighbors
//     //////////////////////////////////////////////////////////////////////
//     for( int k=0 ; k<numNeigh ; k++ ) {
//       //dx,dy
//       dx[k] = xk[k]-X;
//       dy[k] = yk[k]-Y;
//       double distance = std::sqrt(dx[k]*dx[k]+dy[k]*dy[k]);
//       dx[k] /= distance;
//       dy[k] /= distance;
//       //xM,yM
//       if( distance<R || distance<r[k] ) {
// 	std::cerr << "Organism::findNeighborAreas: Cells too close!\n";
// 	std::cerr << i << " " << compartment(i).neighbor(k) << " "
// 		  << distance << " " << R << " " << r[k] << "\n";
// 	//exit(-1);
//       }
//       double b = (R*R-r[k]*r[k]+distance*distance)/(2*distance);
//       xM[k] = X + b*dx[k];
//       yM[k] = Y + b*dy[k];
//     }
//     //Set dx,dy,xM,yM for boundaries (square surrounding the cell)  
//     //////////////////////////////////////////////////////////////////////
//     double rFac=2.0;//The square lines are positioned R*rFac from center
//     //define the directions
//     dx[numNeigh] = 1; dy[numNeigh] = 0;
//     dx[numNeigh+1] = -1; dy[numNeigh+1] = 0;
//     dx[numNeigh+2] = 0; dy[numNeigh+2] = 1;
//     dx[numNeigh+3] = 0; dy[numNeigh+3] = -1;
//     //calculate the xM,yM
//     for( int k=numNeigh ; k<numLine ; k++ ) {
//       xM[k] = X + R*rFac*dx[k];
//       yM[k] = Y + R*rFac*dy[k];
//     }
//     //Calculate vertex positions
//     //////////////////////////////////////////////////////////////////////
//     std::vector<double> vertexX,vertexY;
//     std::vector<int> vertexK1,vertexK2;
//     //Find minimal distance between (X,Y) and (xM,yM)
//     double minD = std::sqrt( (X-xM[0])*(X-xM[0])+(Y-yM[0])*(Y-yM[0]) );
//     int minK=0;
//     for( int k=1 ; k<numLine ; k++ ) {
//       double d = std::sqrt( (X-xM[k])*(X-xM[k])+(Y-yM[k])*(Y-yM[k]) ); 
//       if( d<minD ) {
// 	minD = d;
// 	minK = k;
//       }
//     }
//     //Define next vertex as the intersection in a 'positive' direction 
//     int count=-1;
//     do {
//       count++;
//       vertexK1.push_back(minK);//From minimal distance (first from above)
//       //std::cerr << i << " " << " K1(" << count << ") -> " << minK << "\n";
//       int firstFlag=1;
//       for( int k=0 ; k<numLine ; k++ ) {
// 	if( k != vertexK1[count] && (count==0 || k!=vertexK1[count-1]) ) {
// 	  //Check that lines are not parallell or with diverging t
// 	  if(dx[k]==dx[vertexK1[count]] && dy[k]==dy[vertexK1[count]] ) {
// 	    //std::cerr << "Organism::findNeighborAreas: Parallell lines...\n";
// 	  }
// 	  else if( (dx[k]==0.0 || dy[vertexK1[count]]==0.0) && 
// 		   (dy[k]==0.0 || dx[vertexK1[count]]==0.0) ) {
// 	    //std::cerr << "Organism::findNeighborAreas: Diverging t...\n";
// 	  }
// 	  else {
// 	    //length from point vertexK1 to intersect between 'lines' vertK1,k 
// 	    double d;
// 	    if( count == 0 )
// 	      d = ( dy[k]*(yM[k]-yM[vertexK1[count]]) + 
// 		    dx[k]*(xM[k]-xM[vertexK1[count]]) ) / 
// 		( dx[k]*dy[vertexK1[count]]-dx[vertexK1[count]]*dy[k] );
// 	    else
// 	      d = ( dy[k]*(yM[k]-vertexY[count-1]) + 
// 		    dx[k]*(xM[k]-vertexX[count-1]) ) / 
// 		( dx[k]*dy[vertexK1[count]]-dx[vertexK1[count]]*dy[k] );
	      
// 	    if( d>=0.0 && (firstFlag==1 || d<minD) ) {
// 	      minD = d;
// 	      minK = k;
// 	      firstFlag=0;
// 	    }
// 	  }
// 	}
//       }
//       if( firstFlag==1 ) {
// 	std::cerr << "Organism::findNeighborArea: No vertex defined!\n";
// 	exit(-1);
//       }
//       vertexK2.push_back(minK);
//       //std::cerr << i << " " << " K2(" << count << ") -> " << minK << "\n";
//       if( count==0 ) {
// 	vertexX.push_back( xM[vertexK1[count]] + minD*dy[vertexK1[count]] );
// 	vertexY.push_back( yM[vertexK1[count]] - minD*dx[vertexK1[count]] );
//       }
//       else {
// 	vertexX.push_back( vertexX[count-1] + minD*dy[vertexK1[count]] );
// 	vertexY.push_back( vertexY[count-1] - minD*dx[vertexK1[count]] );
//       }
//     } while( ( vertexK2[count] != vertexK1[0]) && count<2+numLine );
//     int numVertex = vertexX.size();
//     if( numVertex>numLine ) {
//       std::cerr << "Organism::findNeighborArea: " << numVertex 
// 		<< " verteces defined for " << numLine 
// 		<< " potential neighbors for cell " << i << ".\n";
//       exit(-1);
//     }
//     //Check for verteces outside the sphere, and move them into the sphere
//     //Redefining the verteces
//     //////////////////////////////////////////////////////////////////////
//     std::vector<int> vertexK1Tmp,vertexK2Tmp;
//     std::vector<double> vertexXTmp,vertexYTmp;
//     int maxK=numVertex;
//     double rFactor=1.5;
//     for( int k=0 ; k<maxK ; k++ ) {
//       double d = std::sqrt( (vertexX[k]-X)*(vertexX[k]-X)+
// 			    (vertexY[k]-Y)*(vertexY[k]-Y) );
//       if( d<=R*rFactor  ) {//allow for a close exterior point 
// 	vertexK1Tmp.push_back( vertexK1[k] );
// 	vertexK2Tmp.push_back( vertexK2[k] );
// 	vertexXTmp.push_back( vertexX[k] );
// 	vertexYTmp.push_back( vertexY[k] );
// // 	std::cerr << i << " " << k << " " << maxK << " (" << vertexK1[k] 
// // 		  << "," << vertexK2[k] << "," 
// // 		  << compartment(i).neighbor(vertexK1[k]) << ") inside\n";
//       }
//       else {
// // 	std::cerr << i << " " << k << " " << maxK << " (" << vertexK1[k] 
// // 		  << "," << vertexK2[k] << "," 
// // 		  << compartment(i).neighbor(vertexK1[k]) << ") OUTSIDE\n";

// 	int kTmp=k,kOut,kIn;
// 	double dK;
// 	if( k==0 ) {
// 	  do {
// 	    dK = std::sqrt( (xM[vertexK1[kTmp]]-X)*(xM[vertexK1[kTmp]]-X)+
// 			    (yM[vertexK1[kTmp]]-Y)*(yM[vertexK1[kTmp]]-Y) );
// 	    kTmp--;
// 	    if( kTmp<0 ) kTmp=numVertex-1;
// 	    d = std::sqrt( (vertexX[kTmp]-X)*(vertexX[kTmp]-X)+
// 			   (vertexY[kTmp]-Y)*(vertexY[kTmp]-Y) );
// 	  } while( dK>R && d>R*rFactor && kTmp != k);
// 	  if( kTmp != k ) {
// 	    kOut=(kTmp+1)%numVertex;
// 	    if( kOut != 0 ) 
// 	      maxK=kOut;
// 	  }
// 	}
// 	else 
// 	  kOut=k;
// 	kTmp=k;
//       	do {
// 	  dK = std::sqrt( (xM[vertexK2[kTmp]]-X)*(xM[vertexK2[kTmp]]-X)+
// 			  (yM[vertexK2[kTmp]]-Y)*(yM[vertexK2[kTmp]]-Y) );  
// 	  kTmp++;
// 	  if( kTmp>=numVertex ) kTmp=0;
// 	  d = std::sqrt( (vertexX[kTmp]-X)*(vertexX[kTmp]-X)+
// 			 (vertexY[kTmp]-Y)*(vertexY[kTmp]-Y) );
// 	} while( dK>R && d>R*rFactor && kTmp != k); 
// 	if( kTmp != k ) {
// 	  kIn = kTmp==0 ? numVertex-1 : kTmp-1;
// 	  if( k<kIn ) k=kIn;
// 	}
// 	else
// 	  std::cerr << i << " all verteces outside sphere...\n";
// 	//Find the intersections between lines kOut,kIn and the sphere	
// 	//and add the new verteces
// 	//kOut
// 	double a = 2*( dy[vertexK1[kOut]]*(vertexX[kOut]-X)-
// 		       dx[vertexK1[kOut]]*(vertexY[kOut]-Y) ) / 
// 	  ( dx[vertexK1[kOut]]*dx[vertexK1[kOut]]+
// 	    dy[vertexK1[kOut]]*dy[vertexK1[kOut]] );
// 	double b = ( vertexX[kOut]*vertexX[kOut] + 
// 		     vertexY[kOut]*vertexY[kOut] + 
// 		     X*X + Y*Y - R*R - 2*(X*vertexX[kOut]+Y*vertexY[kOut]) ) /
// 	  ( dx[vertexK1[kOut]]*dx[vertexK1[kOut]]+
// 	    dy[vertexK1[kOut]]*dy[vertexK1[kOut]] );
// 	double sqR = a*a/4 - b;
// 	double xV,yV;
// 	if( sqR<0.0 ) {
// 	  std::cerr << "Organism::findNeighborAreas: No intersection between"
// 		    << " line and sphere!\n";
// 	  std::cerr << i << " " << kOut << " " << vertexK1[kOut] << " " 
// 		    << vertexK2[kOut] << " (" << numNeigh << ") " 
// 		    << a << " " << b << " " << sqR << "\n";
// 	  exit(-1);
// 	}      
// 	else if( sqR==0.0 ) {
// 	  double t = -a/2;
// 	  xV = vertexX[kOut]+t*dy[vertexK1[kOut]];
// 	  yV = vertexY[kOut]-t*dx[vertexK1[kOut]];
// 	}
// 	else {
// 	  double t1 = -a/2 + std::sqrt(a*a/4-b);
// 	  double t2 = -a/2 - std::sqrt(a*a/4-b);
// 	  double xV1 = vertexX[kOut]+t1*dy[vertexK1[kOut]];
// 	  double yV1 = vertexY[kOut]-t1*dx[vertexK1[kOut]];
// 	  double xV2 = vertexX[kOut]+t2*dy[vertexK1[kOut]];
// 	  double yV2 = vertexY[kOut]-t2*dx[vertexK1[kOut]];
// 	  double d1 = (xV1-vertexX[kOut])*(xV1-vertexX[kOut]) + 
// 	    (yV1-vertexY[kOut])*(yV1-vertexY[kOut]);
// 	  double d2 = (xV2-vertexX[kOut])*(xV2-vertexX[kOut]) + 
// 	    (yV2-vertexY[kOut])*(yV2-vertexY[kOut]);
// 	  if( d1< d2 ) {
// 	    xV = xV1;
// 	    yV = yV1;
// 	  }
// 	  else {
// 	    xV = xV2;
// 	    yV = yV2;
// 	  }
// 	}	  
// 	vertexK1Tmp.push_back( vertexK1[kOut] );
// 	vertexK2Tmp.push_back( -1 );
// 	vertexXTmp.push_back( xV );
// 	vertexYTmp.push_back( yV );	
// 	//kIn
// 	a = 2*( dy[vertexK2[kIn]]*(vertexX[kIn]-X)-
// 		dx[vertexK2[kIn]]*(vertexY[kIn]-Y) ) / 
// 	  ( dx[vertexK2[kIn]]*dx[vertexK2[kIn]]+
// 	    dy[vertexK2[kIn]]*dy[vertexK2[kIn]] );
// 	b = ( vertexX[kIn]*vertexX[kIn] + 
// 	      vertexY[kIn]*vertexY[kIn] + 
// 	      X*X + Y*Y - R*R - 2*(X*vertexX[kIn]+Y*vertexY[kIn]) ) /
// 	  ( dx[vertexK2[kIn]]*dx[vertexK2[kIn]]+
// 	    dy[vertexK2[kIn]]*dy[vertexK2[kIn]] );
// 	sqR = a*a/4 - b;
// 	if( sqR<0.0 ) {
// 	  std::cerr << "Organism::findNeighborAreas: No intersection between"
// 		    << " line and sphere!\n";
// 	  std::cerr << i << " " << kIn << " " << vertexK1[kIn] << " " 
// 		    << vertexK2[kIn] << " (" << numNeigh << ") " 
// 		    << a << " " << b << " " << sqR << "\n";
// 	  exit(-1);
// 	}      
// 	else if( sqR==0.0 ) {
// 	  double t = -a/2;
// 	  xV = vertexX[kIn]+t*dy[vertexK2[kIn]];
// 	  yV = vertexY[kIn]-t*dx[vertexK2[kIn]];
// 	}
// 	else {
// 	  double t1 = -a/2 + std::sqrt(a*a/4-b);
// 	  double t2 = -a/2 - std::sqrt(a*a/4-b);
// 	  double xV1 = vertexX[kIn]+t1*dy[vertexK2[kIn]];
// 	  double yV1 = vertexY[kIn]-t1*dx[vertexK2[kIn]];
// 	  double xV2 = vertexX[kIn]+t2*dy[vertexK2[kIn]];
// 	  double yV2 = vertexY[kIn]-t2*dx[vertexK2[kIn]];
// 	  double d1 = (xV1-vertexX[kIn])*(xV1-vertexX[kIn]) + 
// 	    (yV1-vertexY[kIn])*(yV1-vertexY[kIn]);
// 	  double d2 = (xV2-vertexX[kIn])*(xV2-vertexX[kIn]) + 
// 	    (yV2-vertexY[kIn])*(yV2-vertexY[kIn]);
// 	  if( d1< d2 ) {
// 	    xV = xV1;
// 	    yV = yV1;
// 	  }
// 	  else {
// 	    xV = xV2;
// 	    yV = yV2;
// 	  }
// 	}	  
// 	vertexK1Tmp.push_back( -1 );
// 	vertexK2Tmp.push_back( vertexK2[kIn] );
// 	vertexXTmp.push_back( xV );
// 	vertexYTmp.push_back( yV );	
//       }
//     }
//     //Copy into real vertex variables
//     vertexK1 = vertexK1Tmp;
//     vertexK2 = vertexK2Tmp;
//     vertexX = vertexXTmp;
//     vertexY = vertexYTmp;
//     //Calculate the areas (distance between vertices)
//     //////////////////////////////////////////////////////////////////////
//     numVertex = vertexK1.size();
//     std::vector<double> area( numVertex );
//     std::vector<int> neigh( numVertex );
//     for( int a=0 ; a<numVertex ; a++ ) {
//       int aa = a>0 ? (a-1) : numVertex-1;
//       area[a] =  std::sqrt( (vertexX[aa]-vertexX[a])*
// 			    (vertexX[aa]-vertexX[a]) +
// 			    (vertexY[aa]-vertexY[a])*
// 			    (vertexY[aa]-vertexY[a]) );
//       if( vertexK1[a]>=0 && vertexK1[a]<numNeigh )
// 	neigh[a] = compartment(i).neighbor(vertexK1[a]);
//       else
// 	neigh[a] = -1;
// //     std::cerr << i << " " << a << " " << vertexK2[a] << " - " << vertexK1[a]
// // 		<< "=" << vertexK2[aa] << " - " << vertexK1[aa] << " " 
// // 		<< area[a] << " " << neigh[a] << "\n";
//     }      
//     //Copy the new neighbour areas into the cells
//     //also copy new neighbor vector...
//     compartment(i).setNeighbor( neigh );
//     compartment(i).setNeighborArea( area );

//     //Calculate the volume of the cell
//     double volume=0.0,u,ux,uy,v,vx,vy,uv,theta;
//     for( int a=0 ; a<numVertex ; a++ ) {
//       int aa = a>0 ? (a-1) : numVertex-1;
//       ux = vertexX[a]-X;
//       uy = vertexY[a]-Y;
//       vx = vertexX[aa]-X;
//       vy = vertexY[aa]-Y;
//       u = std::sqrt( ux*ux+uy*uy );
//       v = std::sqrt( vx*vx+vy*vy );
//       uv = ux*vx+uy*vy;
//       theta = std::acos(uv/(u*v));
//       volume += 0.5* u*v*std::sin(theta);
//     }    
//     compartment(i).setVolume( volume );
    
//     //Do some test printing...
//     //Cells with potential neighbors
// //     for( int k=0 ; k<numNeigh ; k++ ) {
// //       std::cout << i << " 1 " << X << " " << Y << "\n"; 
// //       std::cout << i << " 1 " << xM[k] << " " << yM[k] << "\n"; 
// //       std::cout << i << " 1 " << xk[k] << " " << yk[k] << "\n"; 
// //     }
// //     for( int k=numNeigh ; k<numLine ; k++ ) {
// //       std::cout << i << " 1 " << X << " " << Y << "\n"; 
// //       std::cout << i << " 1 " << xM[k] << " " << yM[k] << "\n"; 
// //     }
// //     std::cout << "\n";
// //     //Verteces
// //     for( int a=0 ; a<numVertex ; a++ ) {
// //       std::cout << i << " 2 " << vertexX[a] << " " 
// // 		<< vertexY[a] << "\n"; 
// //     }
// //     std::cout << i << " 2 " << vertexX[0] << " " 
// // 	      << vertexY[0] << "\n\n";     
// //     int numVertexTmp = vertexXTmp.size();
// //     for( int a=0 ; a<numVertexTmp ; a++ ) {
// //       std::cout << i << " 4 " << vertexXTmp[a] << " " 
// //  		<< vertexYTmp[a] << "\n"; 
// //     }    
// //     std::cout << i << " 4 " << vertexXTmp[0] << " " 
// // 	      << vertexYTmp[0] << "\n"; 
// //     //Sphere
// //     for( int k=0 ; k<numLine ; k++ ) {
// //       std::cout << i << " 3 " << X + R*dx[k] << " " 
// // 		<< Y + R*dy[k] << "\n"; 
// //     }
// //     for( int a=0 ; a<numVertex ; a++ ) {
// //       double dxV=vertexX[a]-X;
// //       double dyV=vertexY[a]-Y;
// //       double factor = 1./std::sqrt( dxV*dxV+dyV*dyV);
// //       dxV *= factor;
// //       dyV *= factor;
// //       std::cout << i << " 3 " << X + R*dxV << " " 
// // 		<< Y + R*dyV << "\n"; 
// //     }
// //     for( int a=0 ; a<numVertexTmp ; a++ ) {
// //       double dxV=vertexXTmp[a]-X;
// //       double dyV=vertexYTmp[a]-Y;
// //       double factor = 1./std::sqrt( dxV*dxV+dyV*dyV);
// //       dxV *= factor;
// //       dyV *= factor;
// //       std::cout << i << " 3 " << X + R*dxV << " " 
// // 		<< Y + R*dyV << "\n"; 
// //     }
    
//   }
// }

// //! Find neighbors to the compartments and puts them in given neighbor matrix
// /*! Simple O(N^2) function for finding neighboring compartments.
//   rFactor gives a factor of the sum of the two cell radiuses for which
//   a threshold is set. If the distance is larger than the threshold,
//   the cells are not considered as neighbors. It returns the total
//   number of neighbors found.
// */
// int Organism::findNeighbors( std::vector< std::vector<double> > &y,
// 			     std::vector< std::vector<int> > &neighbor) {

//   double rFactor=1.2; //UGLY
//   neighbor.resize( numCompartment() );
//   for( int i=0 ; i<neighbor.size() ; i++ )
//       if( neighbor[i].size() )
// 	  neighbor[i].resize(0);
//   double d,r;
//   int numNeigh=0;
  
//   int rIndex = dimension();
//   int eStart = 0;
  
//   for(int i=0 ; i<numCompartment() ; i++ )
//     for(int j=i+1 ; j<numCompartment() ; j++ ) {
//       r = y[i][rIndex] + y[j][rIndex];
//       d=0.;
//       for( int dim=eStart ; dim<rIndex ; dim++ )
// 	d += (y[i][dim]-y[j][dim])*(y[i][dim]-y[j][dim]);
//       d = sqrt( d );
      
//       if( d<= r*rFactor ) {
//         //Add neighbor
//         neighbor[i].push_back(j);
//         neighbor[j].push_back(i);
//         numNeigh++;
//       }
//     }

//   //Copy the new neighbours into the cells (Caveat: Not a good solution)
//   for(int i=0 ; i<numCompartment() ; i++ )
//     compartment(i).setNeighbor( neighbor[i] );
  
//   return numNeigh;
// }

// //!Reads neighbor information from a file.
// /*! This function is typically used when a spatial template without
//   neighbor areas is defined. It reads neighborhood information.*/
// int Organism::readNeighbors( std::string fileNameVal ) {
  
//   int numNeigh=0;
//   //Create and resize temporary neighbor and area vectors
//   //////////////////////////////////////////////////////////////////////
//   std::vector<int> neighbor;
  
//   //Read all values into the temporary matrices
//   //////////////////////////////////////////////////////////////////////
//   const char* fileName=fileNameVal.c_str();  
//   std::ifstream IN( fileName );
//   if( !IN ) {
//     std::cerr << "Organism::readNeighborsWithArea() - "
// 	      << "Cannot open file " << fileName << "\n\n\7";exit(-1);}
  
//   std::cerr << "Organism::readNeighborWithAreas() reading neighborhood file " 
// 	    << fileName << " ... ";
  
//   int tmpInt,K,numPara;
//   double tmpDouble;
//   IN >> tmpInt;//N
//   if( tmpInt != numCompartment() ) {
//     std::cerr << "Organism::readNeighborsWithArea() - "
// 	      << "Wrong number of compartments in file " << fileName 
// 	      << "\n\n\7";
//     std::cerr << tmpInt << " " << numCompartment() << "\n";
//     exit(-1);}
//   IN >> numPara;//number of parameters to read and discard...
  
//   for( int i=0 ; i<numCompartment() ; i++ ) {
//     IN >> tmpInt;//i
//     if( tmpInt != i ) {
//       std::cerr << "Organism::readNeighborsWithArea() - "
// 		<< "Strange compartment number in file " << fileName 
// 		<< "\n\n\7";exit(-1);}
//     IN >> K;//K
//     numNeigh +=K;
//     neighbor.resize(K);
//     for( int k=0 ; k<K ; k++ )
//       IN >> neighbor[k];//neigh
//     for( int pC=0 ; pC<numPara ; pC++ )
//       for( int k=0 ; k<K ; k++ )
// 	IN >> tmpDouble;//parameter values that are disregarded
    
//     //Set the values in the compartment
//     compartment(i).setNeighbor( neighbor );    
//   }
//   return numNeigh/2;
// }

// //!Reads neighbor information from a file.
// /*! This function is typically used when a spatial template is
//   defined. It reads neighborhood information and also intersecting
//   areas between defined neighbors.*/
// int Organism::readNeighborsWithAreas( std::string fileNameVal ) {
  
//   int numNeigh=0;
//   //Create and resize temporary neighbor and area vectors
//   //////////////////////////////////////////////////////////////////////
//   std::vector<int> neighbor;
//   std::vector<double> neighborArea;
  
//   //Read all values into the temporary matrices
//   //////////////////////////////////////////////////////////////////////
//   const char* fileName=fileNameVal.c_str();  
//   std::ifstream IN( fileName );
//   if( !IN ) {
//     std::cerr << "Organism::readNeighborsWithArea() - "
// 	      << "Cannot open file " << fileName << "\n\n\7";exit(-1);}
  
//   std::cerr << "Organism::readNeighborWithAreas() reading neighborhood file " 
// 	    << fileName << " ... ";
  
//   int tmpInt,K;
//   double tmpDouble;
//   IN >> tmpInt;//N
//   if( tmpInt != numCompartment() ) {
//     std::cerr << "Organism::readNeighborsWithArea() - "
// 	      << "Wrong number of compartments in file " << fileName 
// 	      << "\n\n\7";
//     std::cerr << tmpInt << " " << numCompartment() << "\n";
//     exit(-1);}
//   IN >> tmpInt;//number of parameters (should be 1, the area)
//   if( tmpInt != 1 ) {
//     std::cerr << "Organism::readNeighborsWithArea() - "
// 	      << "Wrong number of area variables in file " << fileName 
// 	      << "\n\n\7";exit(-1);}
  
//   for( int i=0 ; i<numCompartment() ; i++ ) {
//     IN >> tmpInt;//i
//     if( tmpInt != i ) {
//       std::cerr << "Organism::readNeighborsWithArea() - "
// 		<< "Strange compartment number in file " << fileName 
// 		<< "\n\n\7";exit(-1);}
//     IN >> K;//K
//     numNeigh +=K;
//     neighbor.resize(K);
//     neighborArea.resize(K);
//     for( int k=0 ; k<K ; k++ )
//       IN >> neighbor[k];//neigh
//     for( int k=0 ; k<K ; k++ )
//       IN >> neighborArea[k];//neighArea
    
//     //Set the values in the compartment
//     compartment(i).setNeighbor( neighbor );    
//     compartment(i).setNeighborArea( neighborArea );    
//   }
//   return numNeigh/2;
// }

// //!Makes the system periodic in Y by adding neighbors between minX/maxX cells
// int Organism::addPeriodicBoundaries(std::vector< std::vector<double> > 
// 				    &yPosR, int dPosition) {
    
//     if( numCompartment() != yPosR.size() || !yPosR.size() ) {
//  	std::cerr << "Organism::addPeriodicBoundaries "
//  		  << "Zero or wrong size of given position vector.\n";
//  	exit(-1);
//     }
//     if( dimension()+numTopology() > yPosR[0].size() ) {
//  	std::cerr << "Organism::addPeriodicBoundaries "
//  		  << "Zero or wrong size of given position vector.\n";
//  	exit(-1);
//     }
//     if( dimension() <= dPosition ) {
//  	std::cerr << "Organism::addPeriodicBoundaries "
//  		  << "Dimension to add periodic boundary conditions wrong.\n";
//  	exit(-1);
//     }
    
    
//     //Find maximal and minimal positions in dimension dPosition
//     double max,min;
//      int rPosition=dimension();
//      for( int i=0 ; i<yPosR.size() ; i++ ) {
// 	 if( i==0 || yPosR[i][dPosition]+yPosR[i][rPosition] > max )
// 	     max = yPosR[i][dPosition]+yPosR[i][rPosition];
// 	 if( i==0 || yPosR[i][dPosition]-yPosR[i][rPosition] < min )
// 	     min = yPosR[i][dPosition]-yPosR[i][rPosition];
//      }
     
//      //Extract potential cells for periodic boundary
//      std::vector<int> potentialLow,potentialHigh;
//      for( int i=0 ; i<yPosR.size() ; i++ ) {
// 	 if( yPosR[i][dPosition]+2*yPosR[i][rPosition] >= max )
// 	     potentialHigh.push_back(i);
// 	 else if( yPosR[i][dPosition]-2*yPosR[i][rPosition] <= min )
// 	     potentialLow.push_back(i);
//      }

//      //Move low cells above top ones
//      double delta=max-min;
//      for( int k=0 ; k<potentialLow.size() ; k++ ) {
// 	 int i=potentialLow[k];
// 	 yPosR[i][dPosition] += delta;
//      }

//      //Find and add new neighbors
//      double rFactor=1.2; //UGLY
//      int numAddedNeighbors=0;
//      for( int k=0 ; k<potentialLow.size() ; k++ ) {
// 	 int i=potentialLow[k];
// 	 for( int m=0 ; m<potentialHigh.size() ; m++ ) {
// 	     int j=potentialHigh[m];
// 	     double distance=0.0;
// 	     for( int d=0 ; d<dimension() ; d++ )
// 		 distance += (yPosR[i][d]-yPosR[j][d])*
// 		     (yPosR[i][d]-yPosR[j][d]);
// 	     distance = sqrt(distance);
// 	     if( distance<=1.2*(yPosR[i][rPosition]+yPosR[i][rPosition]) ) {
// 		 compartment(i).addNeighbor(j);
// 		 compartment(j).addNeighbor(i);		 
// 		 numAddedNeighbors++;
// 	     }
// 	 }
//      }
//      //Move the cells back...
//      for( int k=0 ; k<potentialLow.size() ; k++ ) {
// 	 int i=potentialLow[k];
// 	 yPosR[i][dPosition] -= delta;
//      } 
     
//      return numAddedNeighbors;
// }

// //!Adds membrane neighborhood in a very particular way!
// int Organism::addNeighborsRestricted(std::vector< std::vector<double> > &y,
// 				     int cellMarker,int membraneMarker) {
  

//   if( !y.size() || y[0].size()<=cellMarker || y[0].size()<=membraneMarker ) {
//     std::cerr << "Organism::addNeighborRestricted"
// 	      << " Wrong size of given matrix.\n";
//     exit(-1);
//   }
//   if( y.size() != numCompartment() ) {
//     std::cerr << "Organism::addNeighborRestricted"
// 	      << " Size of given matrix not same as number of compartments.\n";
//     exit(-1);
//   }

//   int numAdded=0;

//   for( int i=0 ; i<y.size() ; i++ ) {

//     //Start adding membrane neighbors from each cell
//     if( y[i][cellMarker]>0.5 ) {
//       for( int k=0 ; k<compartment(i).numNeighbor() ; k++ ) {
// 	int j=compartment(i).neighbor(k);
// 	if( y[j][membraneMarker]<0.5 ) {
// 	  std::cerr << "Organism::addNeighborRestricted"
// 		    << "Cell neighbor not membrane compartment...\n";
// 	  exit(-1);
// 	}
// 	for( int kk=k+1 ; kk<compartment(i).numNeighbor() ; kk++ ) {
// 	  int jj=compartment(i).neighbor(kk);
// 	  if( y[jj][membraneMarker]<0.5 ) {
// 	    std::cerr << "Organism::addNeighborRestricted"
// 		      << "Cell neighbor not membrane compartment...\n";
// 	    exit(-1);
// 	  }
// 	  double d=sqrt( (y[j][0]-y[jj][0])*(y[j][0]-y[jj][0])
// 			 +(y[j][1]-y[jj][1])*(y[j][1]-y[jj][1]) );

// 	  //Very ugly criteria...
// 	  //std::cerr << i << " " << j << " " << jj << " " << d << " " 
// 	  //    << (y[i][2]+y[j][2]+y[jj][2]) << "\n";  
// 	  if( d<1.1*(y[i][2]+y[j][2]+y[jj][2]) ) {
// 	    compartment(j).addNeighbor(jj);
// 	    compartment(jj).addNeighbor(j);		 
// 	    numAdded++;
// 	  }
// 	}
//       }
//     }
//   }
//   return numAdded;
// }

// //!Adds wall neighborhood in a very particular way! Handles periodic bound.
// int Organism::addNeighborsRestricted(std::vector< std::vector<double> > &y,
// 				     int cellMarker,int membraneMarker,
// 				     int wallMarker) {
  

//   if( !y.size() || y[0].size()<=cellMarker || y[0].size()<=membraneMarker 
//       || y[0].size()<=wallMarker ) {
//     std::cerr << "Organism::addNeighborRestricted"
// 	      << " Wrong size of given matrix.\n";
//     exit(-1);
//   }
//   if( y.size() != numCompartment() ) {
//     std::cerr << "Organism::addNeighborRestricted"
// 	      << " Size of given matrix not same as number of compartments.\n";
//     exit(-1);
//   }
  
//   int numAdded=0;

//   for( int i=0 ; i<y.size() ; i++ ) {

//     //Start adding wall neighbors from each cell
//     if( y[i][cellMarker]>0.5 ) {
//       //vector for temporarily storing the wall neighbors
//       std::vector<int> wallTmp( compartment(i).numNeighbor() );
      
//       for( int k=0 ; k<compartment(i).numNeighbor() ; k++ ) {
// 	//Membrane neighbor
// 	int j=compartment(i).neighbor(k);
// 	if( y[j][membraneMarker]<0.5 ) {
// 	  std::cerr << "Organism::addNeighborRestricted"
// 		    << " Cell neighbor not membrane compartment...\n";
// 	  std::cerr << i << " ( " << y[i][cellMarker] << " " 
// 		    << y[i][membraneMarker] << " " << y[i][wallMarker] << " ) "
// 		    << j << " ( " << y[j][cellMarker] << " " 
// 		    << y[j][membraneMarker] << " " << y[j][wallMarker]<<" )\n";
// 	  exit(-1);
// 	}
// 	//Find the wall neighbor
// 	int wCount=0;
// 	for( int l=0 ; l<compartment(j).numNeighbor() ; l++ ) {
// 	  int w=compartment(j).neighbor(l);
// 	  if( y[w][wallMarker]>0.5 ) {
// 	    wallTmp[k]=w;
// 	    wCount++;
// 	  }
// 	}
// 	//Can only exist one and one only
// 	if( wCount != 1 ) {
// 	  std::cerr << "Organism::addNeighborRestricted"
// 		    << " Wrong number of wall neighbors\n";
// 	  exit(-1);
// 	}
//       }
//       //Move wall compartment temporarily if "periodic" (dist to cell large)
//       //Also put the walls in +-xy for later addition of neighbors
//       int xP=-1,xM=-1,yP=-1,yM=-1;
//       int xPCount=0,xMCount=0,yPCount=0,yMCount=0;
//       double cellR=y[i][2];
//       for( int kk=0 ; kk<wallTmp.size() ; kk++ ) {
// 	double xD=y[wallTmp[kk]][0]-y[i][0];
// 	double yD=y[wallTmp[kk]][1]-y[i][1];	

// 	if( fabs(yD)<cellR ) {
// 	  //xP
// 	  if( (xD>cellR && xD<3*cellR) || xD<-3*cellR ) {
// 	    xP=kk;
// 	    xPCount++;
// 	    //std::cerr << "cell " << i << ", " << kk << " added into xP\n";
// 	  }
// 	  //xM
// 	  else {
// 	    xM=kk;
// 	    xMCount++;
// 	    //std::cerr << "cell " << i << ", " << kk << " added into xM\n";
// 	  }
// 	}
// 	if( fabs(xD)<cellR ) {
// 	  //yP
// 	  if( (yD>cellR && yD<3*cellR) || yD<-3*cellR ) {
// 	    yP=kk;
// 	    yPCount++;
// 	    //std::cerr << "cell " << i << ", " << kk << " added into yP\n";
// 	  }
// 	  //yM
// 	  else {
// 	    yM=kk;
// 	    yMCount++;
// 	    //std::cerr << "cell " << i << ", " << kk << " added into yM\n";
// 	  }
// 	}
//       }
//       if( xP==-1 || xM==-1 || yP==-1 || yM==-1 ) {
// 	std::cerr << "Organism::addNeighborRestricted"
// 		  << " Wall positions not well defined\n";
// 	exit(-1);
//       }
//       if( xPCount!=1 || xMCount!=1 || yPCount!=1 || yMCount!=1 ) {
// 	std::cerr << "Organism::addNeighborRestricted"
// 		  << " Wall positions not well defined\n";
// 	exit(-1);
//       }
//       //Add the neighbors
//       compartment(wallTmp[xP]).addNeighbor(wallTmp[yP]);
//       compartment(wallTmp[yP]).addNeighbor(wallTmp[xP]);

//       compartment(wallTmp[xP]).addNeighbor(wallTmp[yM]);
//       compartment(wallTmp[yM]).addNeighbor(wallTmp[xP]);

//       compartment(wallTmp[xM]).addNeighbor(wallTmp[yP]);
//       compartment(wallTmp[yP]).addNeighbor(wallTmp[xM]);

//       compartment(wallTmp[xM]).addNeighbor(wallTmp[yM]);
//       compartment(wallTmp[yM]).addNeighbor(wallTmp[xM]);
//       numAdded+=4;
//     }
//   }
//   return numAdded;
// }

// //!Adds a cell to the right of existing cells w/o updating the neighborhood
// void Organism::addCell1DLattice(std::vector< std::vector<double> > &y,
// 				std::vector< std::vector<double> > &dydt ) {
  
//   int NN=numCompartment();
//   int NNN = NN-1;
//   int N=NN+1;
//   int M=compartment(0).numVariable();
//   int dim=compartment(0).dimension();
  
//   addCompartment( compartment(0) );
//   compartment(NN).setIndex(NN);
  
//   y.resize( N );
//   dydt.resize( N );
//   y[NN].resize( M );
//   dydt[NN].resize( M );

//   //Set correct values 
//   for( int j=0 ; j<M ; j++ ) {
//     if( j==0 )
//       y[NN][j] = y[NNN][j]+2*y[NNN][dim];
//     else if( j<=dim )
//       y[NN][j] = y[NNN][j];
//     else
//       y[NN][j] = 0.5*(y[NNN][j]+y[0][j]);
//     dydt[NN][j] = dydt[NNN][j];
//   } 
// }

// //!Adds cells to the right of existing cells w/o updating the neighborhood
// /*! Adds a column of cells to to the right of a square lattice of
//   known height.*/
// void Organism::addCells2DSquareLattice(std::vector< std::vector<double> > &y,
// 				       std::vector< std::vector<double> > 
// 				       &dydt,
// 				       int height) {
  
//   int NN=numCompartment();
//   //Check that lattice can be square
//   int width = int(NN/height);
//   if( double(NN/height) != width ) {
//     std::cerr << "Organism::addCells2DSquareLattice() Strange lattice size\n";
//     exit(0);
//   }
//   int NNN = NN-height;
//   int N=NN+height;
//   int M=compartment(0).numVariable();
//   int dim=compartment(0).dimension();
  
//   for( int i=NN ; i<N ; i++ ) {
//     addCompartment( compartment(i-height) );
//     compartment(i).setIndex(i);
//   }
//   y.resize( N );
//   dydt.resize( N );
  
//   for( int i=NN ; i<N ; i++ ) {
//     y[i].resize( M );
//     dydt[i].resize( M );
//   }
  
//   //Set correct values 
//   for( int i=NN ; i<N ; i++ ) {
//     int ii= i-height;
//     int iii=i-NN;
//     for( int j=0 ; j<M ; j++ ) {
//       if( j==0 )
// 	y[i][j] = y[ii][j]+2*y[ii][dim];
//       else if( j<=dim )
// 	y[i][j] = y[ii][j];
//       else
// 	y[i][j] = 0.5*(y[ii][j]+y[iii][j]);
//       dydt[i][j] = dydt[ii][j];
//     } 
//   }
// }

// //!Updates the neighborhood from cell indeces assuming 1D lattice and periodic 
// int Organism::getNeighbors1DLatticePeriodic() {

//   std::vector< std::vector<int> > neighbor( numCompartment() );
//   for( int i=0 ; i<neighbor.size() ; i++ )
//     neighbor[i].resize(2);

//   int numNeigh=0;
//   for(int i=0 ; i<numCompartment() ; i++ ) {
//     //Add neighbors
//     if( i==0 )
//       neighbor[i][0] = numCompartment()-1;
//     else
//       neighbor[i][0] = (i-1)%numCompartment();
//     neighbor[i][1] = (i+1)%numCompartment();
//     numNeigh+=2;
//   }

//   //Copy the new neighbours into the cells (Caveat: Not a good solution)
//   for(int i=0 ; i<numCompartment() ; i++ )
//     compartment(i).setNeighbor( neighbor[i] );
  
//   return numNeigh;
// }

// //!Gets the neighbors assuming a 2D lattice of height height
// int Organism::getNeighbors2DSquareLatticePeriodic(int height) {
  
//   int N=numCompartment();
//   int width = int(N/height);
//   //Check that lattice size is square
//   if( double(N/height) != width || height <=1 ) {
//     std::cerr << "Organism::getNeighbors2DSquareLatticePeriodic() "
// 	      << "Strange lattice size\n";
//     exit(0);
//   }
  
//   std::vector< std::vector<int> > neighbor( numCompartment() );
//   int numNeigh=0;
//   int counter=0;
//   for( int j=0 ; j<width ; j++ ) {
//     for( int i=0 ; i<height ; i++ ) {
//       if( i==0 ) {
// 	neighbor[counter].resize( 3 );
// 	numNeigh += 3;
// 	//Add neighbors
// 	if( j==0 )
// 	  neighbor[counter][0] = N-height+i;
// 	else
// 	  neighbor[counter][0] = counter-height;
// 	if( j==width-1 )
// 	  neighbor[counter][1] = i;
// 	else
// 	  neighbor[counter][1] = counter+height;
// 	neighbor[counter][2] = counter+1;
//       }
//       else if( i==height-1 ) {
// 	neighbor[counter].resize( 3 );
// 	numNeigh += 3;
// 	//Add neighbors
// 	if( j==0 )
// 	  neighbor[counter][0] = N-height+i;
// 	else
// 	  neighbor[counter][0] = counter-height;
// 	if( j==width-1 )
// 	  neighbor[counter][1] = i;
// 	else
// 	  neighbor[counter][1] = counter+height;
// 	neighbor[counter][2] = counter-1;
//       }
//       else {
// 	neighbor[counter].resize( 4 );
// 	numNeigh += 4;
// 	//Add neighbors
// 	if( j==0 )
// 	  neighbor[counter][0] = N-height+i;
// 	else
// 	  neighbor[counter][0] = counter-height;
// 	if( j==width-1 )
// 	  neighbor[counter][1] = i;
// 	else
// 	  neighbor[counter][1] = counter+height;

// 	neighbor[counter][2] = counter-1;
// 	neighbor[counter][3] = counter+1;
//       }
//       counter++;
//     }
//   }
  
//   //Copy the new neighbours into the cells (Caveat: Not a good solution)
//   for(int i=0 ; i<numCompartment() ; i++ )
//     compartment(i).setNeighbor( neighbor[i] );
  
//   return numNeigh;
// }

//!Removes a compartment and updates the neighborhood
/*! This function can be used when simulating cell migration (out of
  the system) or cell death.*/
void Organism::removeCompartment(size_t i) {
  
  size_t N=numCompartment();
  size_t NN=N-1;
  std::cerr << "Organism::removeCompartment " << i << "\n";
  //Check index
  assert( i<=NN );
  
  //Remove neighbors pointing to removed compartment 
  for( size_t k=0 ; k<compartment(i).numNeighbor() ; k++ ) {
    int removed = compartment( compartment(i).neighbor(k) ).
      removeNeighbor(i);
    if( removed != 1 ) {
      std::cerr << "Organism::removeCompartment(int) "
		<< "Wrong when removing neighbor index, ( "
		<< removed << " index " << i << " removed from " 
		<< compartment(i).neighbor(k) << "," << numCompartment() 
		<< " )\n";
      for( size_t kk=0 ; kk<compartment(i).numNeighbor() ; kk++ )
	std::cerr << compartment(i).neighbor(kk) << " ";
      std::cerr << "\n";
      for( size_t kk=0 ; kk<compartment(compartment(i).neighbor(k)).numNeighbor()
	     ; kk++ )
	std::cerr << compartment(compartment(i).neighbor(k)).neighbor(kk) 
		  << " ";
      std::cerr << "\n";
      //exit(-1);
    }
  }
  
  //(only if not last compartment)
  if( i != NN ) {
    //Change neighbor index for compartments pointing to last compartment
    for( size_t k=0 ; k<compartment(NN).numNeighbor() ; k++ ) {
      unsigned int moved = compartment( compartment(NN).neighbor(k) ).
	changeNeighborIndex(NN,i); 
      if( moved != 1 ) {
	std::cerr << "Organism::removeCompartment(int) "
		  << "Wrong when moving neighbor index, ( "
		  << moved << " index " << i << " moved from " << NN << " in " 
		  << compartment(NN).neighbor(k) << "," << numCompartment()
		  << " )\n";
	for( size_t kk=0 ; kk<compartment(NN).numNeighbor() ; kk++ )
	  std::cerr << compartment(NN).neighbor(kk) << " ";
	std::cerr << "\n";
	for( size_t kk=0 ; kk<compartment(compartment(NN).neighbor(k)).
	       numNeighbor()
	       ; kk++ )
	  std::cerr << compartment(compartment(NN).neighbor(k)).neighbor(kk) 
		    << " ";
	std::cerr << "\n";
	//exit(-1);
      }
    }
    //Copy last compartment element into i and change index
    setCompartment(i,compartment(NN));
    compartment(i).setIndex(i);
  }
  //Truncate compartment vector
  compartment_.pop_back();  
  assert( numCompartment() == NN );
}

//!Calculates the present derivatives 
/*! This is the main derivatives function used when integrating the
  system. It calls the derivs functions for all its subclasses and
  adds it all up.*/
void Organism::derivs( std::vector< std::vector<double> > &y,
		       std::vector< std::vector<double> > &dydt) {
  
  //Check sizes for y an dydt
  assert( numCompartment() == y.size() );
  assert( numCompartment() == dydt.size() );
  
  //Check compartment indeces since these are used in the derivatives
  for( size_t i=0 ; i<numCompartment() ; i++ )
    assert( i==compartment(i).index() );
  
  //Check species indeces since these are used in the derivatives
  if( numCompartment() )
    for( size_t i=0 ; i<numSpecies() ; i++ )
      assert( i == species(i).index()-compartment(0).speciesStart() );
  
  // Start with Topology variables
  //////////////////////////////////////////////////////////////////////
  //all derivative variables to zero
  if( numTopologyVariable() ) {
    for( size_t i=0 ; i<numCompartment() ; i++ )
      for( size_t j=compartment(i).topologyStart() ; 
	   j<compartment(i).speciesStart() ; j++ )
	dydt[i][j] = 0.;
    //update
    if( topology().numReaction() )
      for( size_t i=0 ; i<numCompartment() ; i++ )
	topology().derivs(compartment(i),y,dydt);
  }
  //All species
  //////////////////////////////////////////////////////////////////////
  //all derivative variables to zero
  if( numSpecies() ) {
    for( size_t i=0 ; i<numCompartment() ; i++ )
      for( size_t j=compartment(i).speciesStart() ; 
	   j<compartment(i).numVariable() ; j++ )
	dydt[i][j] = 0.;
    //update
    for( size_t s=0 ; s<numSpecies() ; s++ ) {
      if( species(s).numReaction() )
	for( size_t i=0 ; i<numCompartment() ; i++ )
	  species(s).derivs(compartment(i),y,dydt);  
    }
  }
  //All additional reactions
  //////////////////////////////////////////////////////////////////////
  if( numReaction() ) {
    for( size_t r=0 ; r<numReaction() ; r++ )
      for( size_t i=0 ; i<numCompartment() ; i++ )
	reaction(r)->derivs(compartment(i),static_cast<size_t>(-1),y,dydt);
  }
}

//!Calculates the present derivatives for mechanical interactions only
/*! This function is used in simulations where the mechanics is
  equilibriated at each time step. It calls the derivs functions for
  all mechanical updates as defined in mechEq.  Caveat: The check that
  mechEq only updates positional variables are left to the user!*/
void Organism::derivsMechanical( std::vector< std::vector<double> > &y,
				 std::vector< std::vector<double> > &dydt,
				 std::vector<size_t> &mechEq,
				 std::vector<size_t> &posVar ) {
  
  // Positional variables
  //////////////////////////////////////////////////////////////////////
  //set all derivative variables to zero
  for( size_t i=0 ; i<numCompartment() ; i++ )
    for( size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      dydt[i][j] = 0.;
    }
  //update
  for( size_t i=0 ; i<numCompartment() ; i++ )
    for( size_t k=0 ; k<mechEq.size() ; k++ ) {
      size_t j=mechEq[k];
      topology().reaction(j)->
	derivs(compartment(i),compartment(i).topologyStart(),y,dydt);
    }
  //topology().derivs(compartment(i),y,dydt);
}

//!Calculates the derivatives for all species flagged in the simFlag vector 
/*! This function is the same as the derivatives function, but it only
  updates a subset of species, assuming that the rest are read from a
  template.*/
void Organism::derivsTemplate( std::vector< std::vector<double> > &y,
			       std::vector< std::vector<double> > &dydt,
			       std::vector<int> &simFlag ) {
  
  //Check sizes for y an dydt
  if( numCompartment() != y.size() || numCompartment() != dydt.size() 
      || numVariable() != simFlag.size() ) {
    std::cerr << "Organism::derivsTemplate() Wrong size of y/dydt/simFlag.\n";
    std::cerr << y.size() << " " << dydt.size() << " " << numCompartment() 
	      << "\n" << simFlag.size() << " " << numVariable() << "\n";
    exit(-1);
  }
  
  //Check compartment indeces since these are used in the derivatives
  for( size_t i=0 ; i<numCompartment() ; i++ )
    if( i != compartment(i).index() ) {
      std::cerr << "Organism::derivs() Wrong index in compartment " << i 
		<< " (" << compartment(i).index() << ")\n";
      exit(-1);
    }
  //Check species indeces since these are used in the derivatives
  if( numCompartment() ) {
    for( size_t i=0 ; i<numSpecies() ; i++ )
      if( i != species(i).index()-compartment(0).speciesStart() ) {
	std::cerr << "Organism::derivs() Wrong index in species " << i << " ("
		  << species(i).index() << ")\n";
	exit(-1);
      }
  }
  //Check size of simFlag vector
  if( numCompartment() ) {
    if( compartment(0).numVariable() != simFlag.size() ) {
      std::cerr << "Organism::derivsTemplate() Wrong size of simFlag "
		<< "vector.\n";
      exit(-1);
    }
  }
  //Check that embedding and topology variables are flagged in simFlag
  size_t speciesStart = numTopologyVariable();
//   if( numCompartment() ) {
//     for( int i=0 ; i<speciesStart ; i++ )
//       if( simFlag[i] ) { 
// 	std::cerr << "Organism::derivsTemplate() Embedding or Topoloogy " 
// 		  << "variable marked for simulation.\n";
// 	exit(-1);
//       }
//   }
  //Update Species marked for simulation
  //////////////////////////////////////////////////////////////////////
  //all derivative variables to zero
  for( size_t j=speciesStart ; 
       j<compartment(0).numVariable() ; j++ )
    if( simFlag[j] )
      for( size_t i=0 ; i<numCompartment() ; i++ )
	dydt[i][j] = 0.;
  //update
  for( size_t s=0 ; s<numSpecies() ; s++ )
    if( simFlag[species(s).index()] )
      for( size_t i=0 ; i<numCompartment() ; i++ )
	species(s).derivs(compartment(i),y,dydt);  

  //All additional reactions
  //////////////////////////////////////////////////////////////////////
  for( size_t r=0 ; r<numReaction() ; r++ )
    for( size_t i=0 ; i<numCompartment() ; i++ )
      reaction(r)->derivs(compartment(i),static_cast<size_t>(-1),y,dydt);
}

//!Checks for all possible compartment changes as divisions and removals.
void Organism::
compartmentChange( std::vector< std::vector<double> > &y,
		   std::vector< std::vector<double> > &dydt,double t) {
  
  for( size_t k=0 ; k<topology().numCompartmentChange() ; k++ ) {
    for( size_t i=0 ; i<numCompartment() ; i++ ) {
      if( topology().compartmentChange(k)->flag(compartment(i),y,dydt) ) {
	topology().compartmentChange(k)->update((*this),compartment(i),
						y,dydt,t);
	if( topology().compartmentChange(k)->numChange()==-1 )
	  i--;
      }
    }
  }
}

//!A simple first implementation of a cell division check
/*! Checks for a compartment division and applies it in a specific
  direction if the check is True. This should really be changed alot
  to also e. g. invlolve more symmetric/assymmetric parameters. This
  function also invloves lots of hard parameter values which is
  awful...
*/
// void Organism::cellDivision( std::vector< std::vector<double> > &y,
// 			     std::vector< std::vector<double> > &dydt,
// 			     double t ) {
  
//   double diffFraction=0.2;//Caveat: not nice to set this hard
//   double PI=3.14159;  
//   for( int i=0 ; i<numCompartment() ; i++ ) {
//     //Caveat: not nice to set this hard
//     double maxR = 0.9;
//     if( y[i][dimension()]>maxR ) {//divide
//       std::cerr << "cell " << i << " divides at time " << t << "\t"
// 		<< dimension() << " " << y[i][dimension()] << " " <<maxR<<"\n";
//       addDivision(i,t);
//       double h=0.3*y[i][dimension()];
//       std::vector<double> direction( dimension() );
      
//       //       //random direction
//       //       //Create a vector in a random direction (uniformly)
//       if( dimension()==3 ) {
// 	double phi = myRandom::Rnd()*2.*PI;
// 	double theta = myRandom::Rnd()*PI;
// 	double sinTheta = sin(theta);
// 	direction[0] = cos(phi)*sinTheta;
// 	direction[1] = sin(phi)*sinTheta;
// 	direction[2] = cos(theta);
//       }
//       else if( dimension()==2 ) {
// 	double phi = myRandom::Rnd()*2*PI;
// 	direction[0] = cos(phi);
// 	direction[1] = sin(phi);
//       }
//       else if( dimension()==1 ) {
// 	if( myRandom::Rnd()>0.5 )
//  	  direction[0]=1;
// 	else
// 	  direction[0]=-1;
//       }
//       else {
// 	std::cerr << "Organism::cellDivide\n"
// 		  << "Wrong dimension for dividing compartment.\n";
// 	exit(-1);
//       }
      
//       //Close to y direction (for toy meristem?)
// //       double eps=0.05*myRandom::Rnd();
// //       if( dimension()==2 ) {
// // 	direction[0]=eps;
// // 	direction[1]=1./eps;
// // 	if( myRandom::Rnd()>0.5 )
// // 	  direction[0]=-eps;
// //       }
// //       else {
// // 	std::cerr << "Organism::cellDivide\n"
// // 		  << "Wrong dimension for dividing compartment.\n";
// // 	exit(-1);
// //       }

//       //At x direction (for line with growth in one end)
// //      if( dimension()==2 ) {
// // 	direction[0]=1.0;
// // 	direction[1]=0.0;
// //       }
// //       else {
// // 	std::cerr << "Organism::cellDivide\n"
// // 		  << "Wrong dimension for dividing compartment.\n";
// // 	exit(-1);
// //       }
      
//       //Divide
//       divide(y,dydt,i,direction,h,diffFraction);      

//     }
//   }
// }

//!Checks for cell division and divides in an orthogonal plane to the position
/*! Same caveats as for the cellDivision() function...
 */
// void Organism::cellDivisionSphereSurface( std::vector< std::vector<double> > 
// 					  &y,
// 					  std::vector< std::vector<double> > 
// 					  &dydt,
// 					  double t) {
  
//   int verbose=1;
//   double maxR=0.9;//Caveat: not nice to set this hard
//   double diffFraction=0.2;//Caveat: not nice to set this hard
//   double hFac=0.3;//Caveat: not nice to set this hard
//   double PI=3.14159;  
//   if( dimension()!=3 ) {
//     std::cerr << "Organism::cellDivisionSphereSurface() "
// 	      << "Only defined for 2D surface in 3D.\n";
//     exit(-1);
//   }
//   for( int i=0 ; i<numCompartment() ; i++ )
//     if( y[i][dimension()]>maxR ) {//divide
//       if( verbose )
// 	std::cerr << "cell " << i << " divides at time " << t << "\t"
// 		  << dimension() << " " << y[i][dimension()] << " " 
// 		  << maxR << "\n";
//       addDivision(i,t);
//       std::vector<double> direction( dimension() );//direction for the cell div
//       std::vector<double> A(dimension());//Position of the cell
//       double R=0.0;
//       for( int dim=0 ; dim<dimension() ; dim++ ) {
// 	A[dim] = y[i][dim];
// 	R += A[dim]*A[dim];
//       }
//       R = std::sqrt(R);
      
//       getRandomSphereSurfaceDirection(A,direction);

//       //Divide
//       double h=hFac*y[i][dimension()];
//       divideArea(y,dydt,i,direction,h,diffFraction);      
      
//       //Project the new positions onto the sphere
//       double norm = 0.0;
//       for( int dim=0 ; dim<dimension() ; dim++ )
// 	norm += y[i][dim]*y[i][dim];
//       norm = R/std::sqrt(norm);
//       for( int dim=0 ; dim<dimension() ; dim++ )
// 	y[i][dim] *=norm;
//       int NN = y.size()-1;
//       norm=0.0;
//       for( int dim=0 ; dim<dimension() ; dim++ )
// 	norm += y[NN][dim]*y[NN][dim];
//       norm = R/std::sqrt(norm);
//       for( int dim=0 ; dim<dimension() ; dim++ )
// 	y[NN][dim] *=norm;      
//     }
// }

//!Checks for cell division and divides in a sphere(z>0)/cylinder surface 
/*! Same caveats as for the cellDivision() function...
 */
// void Organism::
// cellDivisionSphereCylinderSurface( std::vector< std::vector<double> > 
// 				   &y,
// 				   std::vector< std::vector<double> > 
// 				   &dydt,
// 				   double t) {
  
//   int verbose=1;
//   double diffFraction=0.2;//Caveat: not nice to set this hard
//   double hFac=0.3;//Caveat: not nice to set this hard
//   double PI=3.14159;  
//   if( dimension()!=3 ) {
//     std::cerr << "Organism::cellDivisionSphereCylinderSurface() "
// 	      << "Only defined for 2D surface in 3D.\n";
//     exit(-1);
//   }
//   for( int i=0 ; i<numCompartment() ; i++ ) {
//     double maxR=0.9;//Caveat: not nice to set this hard
//     // For larger cells lower down on a circle
//     //////////////////////////////////////////////////////////////////////
//     double R=7.0;
//     //if( y[i][2]<0.0 ) 
//     //maxR=1.0;
//     //else if( y[i][2]<0.7*R ) 
//     //maxR = std::sqrt(y[i][0]*y[i][0]+y[i][1]*y[i][1])/R;
//     if( y[i][2]<0.7*R ) 
//       maxR=1000.0;
//     //////////////////////////////////////////////////////////////////////
//     if( y[i][dimension()]>maxR ) {//divide
//       if( verbose ) {
// 	std::cerr << "cell " << i << " divides at time " << t << "\t"
// 		  << dimension() << " r_cell=" << y[i][dimension()] << " (" 
// 		  << maxR << ") z=" << y[i][2] << " ";
// 	if( y[i][2]>0.0 ) 
// 	  std::cerr << "R_sphere=" << std::sqrt(y[i][0]*y[i][0]+
// 						y[i][1]*y[i][1]
// 						+y[i][2]*y[i][2]) << "\n";
// 	else
// 	  std::cerr << "R_cylinder=" << std::sqrt(y[i][0]*y[i][0]+
// 						  y[i][1]*y[i][1])
// 		    << " shpereCylinderSurface\n";
//       }
//       addDivision(i,t);
//       std::vector<double> direction( dimension() );//direction for the cell div
//       std::vector<double> A(dimension());//Position of the cell
//       double R=0.0;
//       int zCol=2;
//       for( int dim=0 ; dim<dimension() ; dim++ )
// 	A[dim] = y[i][dim];
//       if( y[i][zCol]>0.0 )
// 	R = std::sqrt( A[0]*A[0]+A[1]*A[1]+A[2]*A[2] );
//       else
// 	R = std::sqrt( A[0]*A[0]+A[1]*A[1] );
//       if( y[i][2]>0.0 )
// 	getRandomSphereSurfaceDirection(A,direction);
//       else
// 	getRandomCylinderSurfaceDirection(A,direction);
      
//       //std::cerr << "dx dy dz  " << direction[0] << " " << direction[1]
//       //	<< " " << direction[2] << " ";
//       //Divide
//       double h=hFac*y[i][dimension()];
//       divideArea(y,dydt,i,direction,h,diffFraction);      
//       int NN = y.size()-1;
      
//       if( verbose ) {
// 	if( y[i][zCol]>0.0 )
// 	  std::cerr << "R(i)_sphere=" 
// 		    << std::sqrt( y[i][0]*y[i][0]+y[i][1]*y[i][1]+
// 				  y[i][2]*y[i][2] ) << " ";
// 	else
// 	std::cerr << "R(i)_cylinder=" 
// 		  << std::sqrt( y[i][0]*y[i][0]+y[i][1]*y[i][1] ) << " ";
// 	if( y[NN][zCol]>0.0 )
// 	  std::cerr << "R(NN)_sphere=" 
// 		    << std::sqrt( y[NN][0]*y[NN][0]+y[NN][1]*y[NN][1]+
// 				  y[NN][2]*y[NN][2] ) << "\n";
// 	else
// 	  std::cerr << "R(NN)_cylinder=" 
// 		    << std::sqrt( y[NN][0]*y[NN][0]+y[NN][1]*y[NN][1] ) 
// 		    << "\n";
//       }
//       //Project the new positions onto the sphere/cylinder surface
//       double normI=R,normNN=R;
//       if( y[i][zCol]>0.0 )
// 	normI /= std::sqrt( y[i][0]*y[i][0]+y[i][1]*y[i][1]+y[i][2]*y[i][2] );
//       else
// 	normI /= std::sqrt( y[i][0]*y[i][0]+y[i][1]*y[i][1] );
//       if( y[NN][zCol]>0.0 )
// 	normNN /= std::sqrt( y[NN][0]*y[NN][0]+y[NN][1]*y[NN][1]+
// 			     y[NN][2]*y[NN][2] );
//       else
// 	normNN /= std::sqrt( y[NN][0]*y[NN][0]+y[NN][1]*y[NN][1] );

//       for( int dim=0 ; dim<zCol ; dim++ ) {
// 	y[i][dim] *=normI;
// 	y[NN][dim] *=normNN;
//       }
//       if( y[i][zCol]>0.0 )
// 	y[i][zCol] *=normI;
//       if( y[NN][zCol]>0.0 )
// 	y[NN][zCol] *=normNN;      
//     }
//   }
// }

//! Divides a compartment along a given direction
/*! Divides a compartment. A new daughter cell is created and moved length h
  from the original cell in direction direction. The cell's mass is
  divided into two with a random fractional "disturbance"
  diffFraction. The dimension is used to be sure that the radius gets the
  correct value. ( 3D: mass \propto r^3 , 2D: mass \propto r^2 )
*/
// void Organism::divide(std::vector< std::vector<double> > &y,
//                       std::vector< std::vector<double> > &dydt,int i,
//                       std::vector<double> &direction,double h,
//                       double diffFraction) {
  
//   int NN=numCompartment();
//   int N=NN+1;
//   int M=compartment(i).numVariable();
//   int dim=compartment(i).dimension();
//   if( direction.size() != dim ) {
//     std::cerr << "Organism::divide(...) direction dimension does not agree "
// 	      << "with compartment dimension.\n";
//     exit(-1);
//   }
  
//   addCompartment( compartment(i) );
//   compartment(NN).setIndex(NN);
  
//   y.resize( N );
//   dydt.resize( N );
//   y[NN].resize( M );
//   dydt[NN].resize( M );
//   for( int j=0 ; j<M ; j++ ) {
//     y[NN][j] = y[i][j];
//     dydt[NN][j] = dydt[i][j];
//   } 
//   //Change the direction vector elements to an vector of length h
//   double norm=0.;
//   for( int d=0 ; d<direction.size() ; d++ )
//     norm += direction[d]*direction[d];
//   if( norm<=0 ) {
//     std::cerr << "Organism::divide() No length on directional vector!\n";
//     exit(-1);
//   }
//   norm = sqrt( norm );
//   h = h/norm;//normalizing AND adjusting length at the same time
//   for( int d=0 ; d<direction.size() ; d++ )
//     direction[d] *= h;
  
//   //Move the mother and daughter symmetrically (only in y)
//   for( int d=0 ; d<direction.size() ; d++ ) {
//     y[i][d] += 0.5*direction[d];
//     y[NN][d] -= 0.5*direction[d];
//   }

//   //Divide mass with two and add some diff (only in y values)
//   double newMass = pow(y[i][dim],dim)/2.;
//   double eps = diffFraction*myRandom::Rnd()*newMass;
//   if( eps>newMass ) eps = newMass;//Prevent negative mass...
//   if( myRandom::Rnd() > 0.5 ) {
//     y[i][dim] = pow(newMass+eps,1./double(dim));
//     y[NN][dim] = pow(newMass-eps,1./double(dim));
//   }
//   else {
//     y[i][dim] = pow(newMass-eps,1./double(dim));
//     y[NN][dim] = pow(newMass+eps,1./double(dim));
//   }
  
//   //Add link relationship between mother/daughter
//   //assume this is done directly after divisions!!!
// }

//! Divides a compartment along a given direction
/*! Divides a compartment. A new daughter cell is created and moved length h
  from the original cell in direction direction. The cell's mass is
  divided into two with a random fractional "disturbance"
  diffFraction. The dimension two is used (2D: mass \propto r^2 )
*/
// void Organism::divideArea(std::vector< std::vector<double> > &y,
//                       std::vector< std::vector<double> > &dydt,int i,
//                       std::vector<double> &direction,double h,
//                       double diffFraction) {
  
//   int NN=numCompartment();
//   int N=NN+1;
//   int M=compartment(i).numVariable();
//   int dim=compartment(i).dimension();
//   if( dim<2 ) {
//     std::cerr << "Organism::divideArea() dimension needs to be at least 2.\n";
//     exit(-1);
//   }
//   if( direction.size() != dim ) {
//     std::cerr << "Organism::divideArea() direction dimension does not agree "
// 	      << "with compartment dimension.\n";
//     exit(-1);
//   }
  
//   addCompartment( compartment(i) );
//   compartment(NN).setIndex(NN);
  
//   y.resize( N );
//   dydt.resize( N );
//   y[NN].resize( M );
//   dydt[NN].resize( M );
//   for( int j=0 ; j<M ; j++ ) {
//     y[NN][j] = y[i][j];
//     dydt[NN][j] = dydt[i][j];
//   } 
//   //Change the direction vector elements to an vector of length h
//   double norm=0.;
//   for( int d=0 ; d<direction.size() ; d++ )
//     norm += direction[d]*direction[d];
//   if( norm<=0 ) {
//     std::cerr << "Organism::divideArea() No length on directional vector!\n";
//     exit(-1);
//   }
//   norm = sqrt( norm );

//   h = h/norm;//normalizing AND adjusting length at the same time
//   for( int d=0 ; d<direction.size() ; d++ )
//     direction[d] *= h;
  
//   //Move the mother and daughter symmetrically (only in y)
//   for( int d=0 ; d<direction.size() ; d++ ) {
//     y[i][d] += 0.5*direction[d];
//     y[NN][d] -= 0.5*direction[d];
//   }
  
//   //Divide mass with two and add some diff (only in y values)
//   double newMass = y[i][dim]*y[i][dim]/2.;
//   double eps = diffFraction*myRandom::Rnd()*newMass;
//   if( eps>newMass ) eps = newMass;//Prevent negative mass...
//   if( myRandom::Rnd() > 0.5 ) {
//     y[i][dim] = std::sqrt(newMass+eps);
//     y[NN][dim] = std::sqrt(newMass-eps);
//   }
//   else {
//     y[i][dim] = std::sqrt(newMass-eps);
//     y[NN][dim] = std::sqrt(newMass+eps);
//   }
  
//   //Add link relationship between mother/daughter
//   //assume this is done directly after divisions!!!
// }

//!Defines a random vector in the plane orthogonal to A and puts it in d
// void Organism::getRandomSphereSurfaceDirection(std::vector<double> &A,
// 					       std::vector<double> &d) {
  
//   if( A.size() != 3 ) {
//     std::cerr << "void Organism::getRandomSphereSurfaceDirection() "
// 	      << "Assumes three dimensions!\n";
//     exit(-1);
//   }
//   if( d.size() != A.size() )
//     d.resize( A.size() );
  
//   double eps = 0.001;
//   double PI=3.141592654;
//   std::vector<double> B(A.size()),r(A.size()),
//     W(A.size()),t(A.size());
//   double R=0.0;
//   for( int dim=0 ; dim<A.size() ; dim++ )
//     R += A[dim]*A[dim];
//   R = std::sqrt(R);
//   //Pick a random vector not parallell to A
//   double AB;
//   do {
//     AB=0.0;
//     for( int dim=0 ; dim<A.size() ; dim++ ) {
//       B[dim] = myRandom::Rnd();
//       AB += A[dim]*B[dim];
//     }
//   } while( std::fabs(R*R-AB)<eps );
//   //Get a vector orthogonal to A (from B)
//   double fac=AB/(R*R);
//   double norm=0.0;
//   for( int dim=0 ; dim<A.size() ; dim++ ) {
//     W[dim] = B[dim] - fac*A[dim];
//     norm += W[dim]*W[dim];
//   }
//   norm = 1./std::sqrt(norm);
//   double AW=0.0;
//   for( int dim=0 ; dim<A.size() ; dim++ ) {
//     W[dim] *= norm;
//     AW += A[dim]*W[dim];
//   }
//   //Make sure A and W are orthogonal 
//   if( AW>eps ) {
//     std::cerr << "Organism::getRandomSphereSurfaceDirection() A,W vectors "
// 	      << "not orthogonal\n";
//     exit(-1);
//   }
//   //Define r
//   for( int dim=0 ; dim<A.size() ; dim++ )
//     r[dim] = A[dim]+W[dim];
//   //Create t as a random rotation of r (around A)
//   // t = r*cos(phi)+A(1-cos(phi)+(rxA/R)sin(phi))
//   double phi = 2*PI*myRandom::Rnd();
//   t[0] = (r[1]*A[2]-r[2]*A[1])/R;
//   t[1] = (r[2]*A[0]-r[0]*A[2])/R;
//   t[2] = (r[0]*A[1]-r[1]*A[0])/R;
//   for( int dim=0 ; dim<A.size() ; dim++ )
//     t[dim] = A[dim] + (r[dim]-A[dim])*cos(phi) + t[dim]*sin(phi);
  
//   //Define the direction vector
//   double Ad=0.0;
//   for( int dim=0 ; dim<A.size() ; dim++ ) {
//     d[dim] = t[dim]-A[dim];
//     Ad += A[dim]*d[dim];
//   }
//   //Make sure A and direction are orthogonal 
//   if( Ad>eps ) {
//     std::cerr << "Organism::cellDivisionSphereSurfaceDirection() "
// 	      << "A and d vectors not orthogonal\n";
//     exit(-1);
//   }
// }

//!Defines a random vector on a cylinder where z is assumed along the length
/*! It unfolds the cylinder, picks a random direction in the z,rPhi
plane and then converts back to x and y.
*/
// void Organism::getRandomCylinderSurfaceDirection(std::vector<double> &A,
// 						 std::vector<double> &d) {
  
//   if( A.size() != 3 ) {
//     std::cerr << "void Organism::getRandomCylinderSurfaceDirection() "
// 	      << "Assumes three dimensions!\n";
//     exit(-1);
//   }
//   if( d.size() != A.size() )
//     d.resize( A.size() );
  
//   double PI=3.141592654;
//   double R=std::sqrt( A[0]*A[0] + A[1]*A[1] );
//   double phi = std::atan( std::fabs(A[1]/A[0]) );
//   if( A[0]>0 && A[1]<0 )
//     phi = 2*PI-phi;
//   else if( A[0]<0 && A[1]>0 )
//     phi = PI-phi;
//   else if( A[0]<0 && A[1]<0 )
//     phi = PI+phi;
  
//   double theta = 2*PI*myRandom::Rnd();
//   d[2] = std::sin(theta);
//   double dPhi = std::cos(theta)/R;
//   //convert dPhi to dx,dy
//   d[0] = R*( std::cos(phi+dPhi) - std::cos(phi) );
//   d[1] = R*( std::sin(phi+dPhi) - std::sin(phi) );
// }

//!Used in a growing simulations where cells outside a radius is removed
// void Organism::removeCellsOutsideSphere( double R,
// 					 std::vector< std::vector<double> > &y,
// 					 std::vector< std::vector<double> > 
// 					 &dydt,
// 					 double t) {
  
//   for( int i=0 ; i<numCompartment() ; i++ ) {
//     double r=0.0;
//     for( int d=0 ; d<dimension() ; d++ )
//       r += y[i][d]*y[i][d];
//     r = sqrt(r);
//     if( r>R ) {//remove
//       //Remove the compartment and update neighborhood in Organism
//       removeCompartment(i);
//       //Remove the lines in y and dydt.
//       int NN=numCompartment();//(one compartment already removed)
//       y[i] = y[NN];
//       dydt[i] = dydt[NN];
//       y.pop_back();
//       dydt.pop_back();
//       i--;//Since the last one is moved into i and needs to be checked
//     }    
//   }
// }

//!Used in a growing simulations where cells below zMin are removed
// void Organism::removeCellsBelowZ( double zMin,
// 				  std::vector< std::vector<double> > &y,
// 				  std::vector< std::vector<double> > 
// 				  &dydt,
// 				  double t) {
  
//   for( int i=0 ; i<numCompartment() ; i++ ) {
//     double z = y[i][dimension()-1];
//     if( z<zMin ) {//remove
//       //Remove the compartment and update neighborhood in Organism
//       removeCompartment(i);
//       //Remove the lines in y and dydt.
//       int NN=numCompartment();//(one compartment already removed)
//       y[i] = y[NN];
//       dydt[i] = dydt[NN];
//       y.pop_back();
//       dydt.pop_back();
//       i--;//Since the last one is moved into i and needs to be checked
//     }    
//   }
// }

//!Scales all space variables such as positions, areas and volumes
/*! The function scales positions (and lengths) with the factor, areas
  with the factor to the power (dimension-1), and volumes with the
  factor to the power dimension. It is used when a template of 'wrong'
  units is used.*/
void Organism::scaleSpace( double factor) {

  if( factor<=0.0 ) {
    std::cerr << "Organism::scaleSpace - Wrong scaling factor given!\n";
    exit(-1);
  }

  for( size_t i=0 ; i<numCompartment() ; i++ ) {
    unsigned int dim=numDimension();
    double factor2 = pow(factor,dim-1);
    double factor3 = pow(factor,dim);
    
    //Scale positions
    for( unsigned int j=0 ; j<dim ; j++ )
      compartment(i).setVariable(j,factor*compartment(i).variable(j));
    //Scale the volume
    compartment(i).setVariable(dim,factor3*compartment(i).variable(dim));
    
    //Scale possible neighbor areas
    if( compartment(i).numNeighborArea() ) {
      for( unsigned int j=0 ; j<compartment(i).numNeighborArea() ; j++ )
	compartment(i).
	  setNeighborArea(j,factor2*compartment(i).neighborArea(j));
    }
  }
}

//!Finds peaks in a variable using gradient ascent
unsigned int Organism::
findPeaksGradientAscent(const std::vector< std::vector< double > > &y,
			size_t col, std::vector<size_t> &cellMax,
			std::vector<size_t> &flag ) {
  
  size_t N = numCompartment();
  //Check size of y
  assert( y.size() == N );
  assert( y[0].size()>col );
  if( cellMax.size() )
    cellMax.resize(0);
  if( flag.size() != N ) flag.resize(N);
  for( size_t i=0 ; i<N ; i++ )
    flag[i]=0;
  
  std::vector<size_t> cellTmp;//Values before threshold check
  std::vector<unsigned int> numTmp;//times cellTmp been visited
  std::vector<size_t> walkTmp;//positions for a walk (start point)
  
  size_t count=1;
  //Find the maxima from each pixel
  for( size_t iStart=0 ; iStart<numCompartment() ; iStart++ ) {
    size_t i=iStart;
    double value,newValue;
    walkTmp.resize(1);
    walkTmp[0]=i;
    //find the max by walking uphill (greedy)
    if( !flag[i] ) {
      do {
	newValue=value=y[i][col];
	size_t newI=i;
	//Check all neighboring cells
	for(size_t k=0 ; k<compartment(i).numNeighbor() ; k++ ) {
	  size_t j = compartment(i).neighbor(k);
	  if( y[j][col]>newValue ) {
	    newValue=y[j][col];
	    newI=j;
	  }
	}
	i=newI;
	walkTmp.push_back( i );
      } while( newValue>value && !flag[i] );
    }
    //Collect the path data and add one visit for the maximum
    if( !flag[i] ) { //new maximum
      cellTmp.push_back( i );
      numTmp.push_back(1);
      unsigned int n=count++;//cellTmp.size();
      for( size_t a=0 ; a<walkTmp.size() ; a++ )
	flag[ walkTmp[a] ] = n;
    }
    else { //old maximum or background
      size_t n = flag[i];
      for( size_t a=0 ; a<walkTmp.size() ; a++ )
	flag[ walkTmp[a] ] = n;
      if( flag[i]>0 )//old maxima
	numTmp[n-1]++;
    }
  }
  //No threshold checking...
  unsigned int threshold = 1;
  double valThreshold = 0.0;
  //Get the maxima visited more than threshold times and with an intensity
  //value higher than threshold
  std::vector<int> clusterNum;
  for( size_t n=0 ; n<cellTmp.size() ; n++ )
    if( numTmp[n]>=threshold && y[ cellTmp[n] ][col]>valThreshold ) {
      cellMax.push_back( cellTmp[n] ); 
      clusterNum.push_back( n+1 );
    }
  
  //Save the basins of attraction
  //   boa.resize( cellMax.size() );
  //   for( int i=0 ; i<numCompartment() ; i++ )
  //     for( int n=0 ; n<cellMax.size() ; n++ )
  //       if( flag[i] == clusterNum[n] )
  // 	boa[n].push_back(i);
  
  return static_cast<unsigned int>(cellMax.size());
}

//!Prints the parameters in the specific model file format
void Organism::printParameter(std::ostream &os) const {

  os << id() << " " << numTopology() << " " << numSpecies() << " " 
     << numReaction() << " " << numNeighborhood() << "\n\n";

  //Topology
  //////////////////////////////////////////////////////////////////////
  if( numTopology() ) {
    os << topology_.id() << " " << numTopologyVariable() << " " 
       << topology_.numReaction() << " " 
       << topology_.numCompartmentChange() << "\n";
    //reactions
    for( size_t i=0 ; i<topology_.numReaction() ; i++ ) {
      os << topology_.reaction(i)->id() << " " 
	 << topology_.reaction(i)->numParameter() << " " 
	 << topology_.reaction(i)->numVariableIndexLevel() << " ";
      for( size_t k=0 ; k<topology_.reaction(i)->numVariableIndexLevel() ; k++ )
	os << topology_.reaction(i)->numVariableIndex(k) << " ";
      os << "\n";
      for( size_t j=0 ; j<topology_.reaction(i)->numParameter() ; j++ )
	os << topology_.reaction(i)->parameter(j) << " ";
      os << "\n";
      for( size_t k=0 ; k<topology_.reaction(i)->numVariableIndexLevel() ; k++ ) {
	for( size_t j=0 ; j<topology_.reaction(i)->numVariableIndex(k) ; j++ )
	  os << topology_.reaction(i)->variableIndex(k,j) << " ";
	os << "\n";
      }
    }
    //compartmentChanges
    for( size_t i=0 ; i<topology_.numCompartmentChange() ; i++ ) {
      os << topology_.compartmentChange(i)->id() << " " 
	 << topology_.compartmentChange(i)->numParameter() << " " 
	 << topology_.compartmentChange(i)->numVariableIndexLevel() << " ";
      for( size_t k=0 ; k<topology_.compartmentChange(i)->numVariableIndexLevel() ; k++ )
	os << topology_.compartmentChange(i)->numVariableIndex(k) << " ";
      os << "\n";
      for( size_t j=0 ; j<topology_.compartmentChange(i)->numParameter() ; j++ )
	os << topology_.compartmentChange(i)->parameter(j) << " ";
      os << "\n";
      for( size_t k=0 ; k<topology_.compartmentChange(i)->numVariableIndexLevel() ; k++ ) {
	for( size_t j=0 ; j<topology_.compartmentChange(i)->numVariableIndex(k) ; j++ )
	  os << topology_.compartmentChange(i)->variableIndex(k,j) << " ";
	os << "\n";
      }
    }
    os << "\n";
  }
  //All species
  //////////////////////////////////////////////////////////////////////
  for( size_t s=0 ; s<numSpecies() ; s++ ) {
    os << species(s).id() << " " << species(s).index() << " "
       << species(s).numReaction() << "\n";
    //reactions
    for( size_t i=0 ; i<species(s).numReaction() ; i++ ) {
      os << species(s).reaction(i)->id() << " " 
	 << species(s).reaction(i)->numParameter() << " " 
	 << species(s).reaction(i)->numVariableIndexLevel() << " ";
      for( size_t k=0 ; k<species(s).reaction(i)->numVariableIndexLevel() ; k++ )
	os << species(s).reaction(i)->numVariableIndex(k) << " ";
      os << "\n";      
      for( size_t j=0 ; j<species(s).reaction(i)->numParameter() ; j++ )
	os << species(s).reaction(i)->parameter(j) << " ";
      os << "\n";
      for( size_t k=0 ; k<species(s).reaction(i)->numVariableIndexLevel() ; k++ ) {
	for( size_t j=0 ; j<species(s).reaction(i)->numVariableIndex(k) ; j++ )
	  os << species(s).reaction(i)->variableIndex(k,j) << " ";
	os << "\n";
      }
    }
    os << "\n";
  }
  //Additional reactions
  //////////////////////////////////////////////////////////////////////
  for( size_t i=0 ; i<numReaction() ; i++ ) {
    os << reaction(i)->id() << " " << reaction(i)->numParameter() << " " 
       << reaction(i)->numVariableIndexLevel() << " ";
    for( size_t k=0 ; k<reaction(i)->numVariableIndexLevel() ; k++ )
      os << reaction(i)->numVariableIndex(k) << " ";
    os << "\n";      
    for( size_t j=0 ; j<reaction(i)->numParameter() ; j++ )
      os << reaction(i)->parameter(j) << " ";
    os << "\n";
    for( size_t k=0 ; k<reaction(i)->numVariableIndexLevel() ; k++ ) {
      for( size_t j=0 ; j<reaction(i)->numVariableIndex(k) ; j++ )
	os << reaction(i)->variableIndex(k,j) << " ";
      os << "\n";
    }
  }
  //Neighborhood
  //////////////////////////////////////////////////////////////////////
  if( numNeighborhood() ) {
    os << neighborhood().id() << " " 
       << neighborhood().numParameter()
       << "\n";
    for( size_t i=0 ; i<neighborhood().numParameter() ; i++ )
      os << neighborhood().parameter(i) << " ";
    os << "\n";
    for( size_t k=0 ; k<neighborhood().numVariableIndexLevel() ; k++ ) {
      for( size_t j=0 ; j<neighborhood().numVariableIndex(k) ; j++ )
	os << neighborhood().variableIndex(k,j) << " ";
      os << "\n";      
    }
    //os << neighborhood().inFile() << "\n";
  }
} 

//!Prints the current values of the compartment variables
void Organism::printVariable(std::ostream &os) const {

  os << numCompartment() << " " << numVariable() << "\n";

  for( size_t i=0 ; i<numCompartment() ; i++ ) {
    for( size_t j=0 ; j<numVariable() ; j++ )
      os << compartment(i).variable(j) << " ";
    os << "\n";
  }
}

//!Prints the current neighborhood variables
void Organism::printNeighbor(std::ostream &os) const {
  
  for( size_t i=0 ; i<numCompartment() ; i++ ) {
    os << compartment(i).index() << " ";
    for( size_t k=0 ; k<compartment(i).numNeighbor() ; k++ ) {
      os << compartment(i).neighbor(k) << " ";
      if( compartment(i).numNeighborArea()>k )
	os << "(" << compartment(i).neighborArea(k) << ") ";
    }
    os << "\n";
  }
}
