/**
 * Filename     : organism.h
 * Description  : A class describing an organism 
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: organism.h,v 1.15 2006/03/18 12:23:08 henrik Exp $
 */
#ifndef ORGANISM_H
#define ORGANISM_H

#include<vector>
#include<string>
#include<iostream>
#include<fstream>

#include"compartment.h"
#include"topology.h"
#include"species.h"
#include"baseReaction.h"
#include"baseCompartmentNeighborhood.h"

//! A class describing properties of an organism (defining the complete model)
/*!The Organism handles the update and information of the topology and
  species (molecules) within each compartment (e.g. cell). It is the
  'top' class for the defined model. It is here reactions updating
  more than one species are defined.*/ 
class Organism {
  
 private:
  
  std::string id_;           
  std::vector<Compartment> compartment_;
  std::vector<int> divisionIndex_;
  std::vector<double> divisionTime_;
  Topology topology_;
  unsigned int numTopology_;
  std::vector<Species> species_;  
  std::vector<BaseReaction*> reaction_;
  std::vector<double*> parameter_;
  BaseCompartmentNeighborhood *neighborhood_;
  unsigned int numNeighborhood_;

 public:
  
  Organism();
  Organism( const Organism & organismCopy );
  Organism( Topology* topologyVal, std::vector<Species> &speciesVal,
	    std::string idValue="", std::string initFile="");
  Organism( const char *modelFile, const char *initFile="", int verbose=0 );
  Organism( std::string modelFile, std::string initFile="", int verbose=0 );
  
  ~Organism();
  
  //Organism & operator=( const Organism & organismCopy );
  
  // Get values
  inline std::string id() const;
  inline size_t numVariable() const;  
  inline size_t numCompartment() const;  
  inline size_t numDivision() const;  
  inline unsigned int numDimension() const;
  inline unsigned int numTopology() const;
  inline size_t numTopologyVariable() const;
  inline size_t numSpecies() const;
  inline size_t numReaction() const;
  inline unsigned int numNeighborhood() const;
  inline const Compartment &compartment(size_t i) const;
  inline Compartment &compartment(size_t i);
  inline int divisionIndex(size_t i) const;
  inline double divisionTime(size_t i) const;
  inline const Topology &topology() const;
  inline Topology &topology();
  inline Topology* topologyPointer();
  inline const Species &species(size_t i) const;
  inline Species &species(size_t i);
  inline BaseReaction* reaction(size_t i) const;
  inline const std::vector<BaseReaction*> reaction() const;
  inline const BaseCompartmentNeighborhood &neighborhood() const;
  inline BaseCompartmentNeighborhood &neighborhood();
  inline BaseCompartmentNeighborhood* neighborhoodPointer();
  inline size_t numParameter() const;
  inline double parameter(size_t i) const;
  
  // Set values
  inline void setId(std::string value);
  inline void setCompartment(size_t i,Compartment &value);
  inline void addCompartment(Compartment &value);
  inline void addDivision(int indexValue,double timeValue);
  inline void setTopology(Topology &value);
  inline void setNumTopology(unsigned int value);
  inline void setSpecies(size_t i,Species &value);
  inline void addSpecies(Species &value);
  inline void addSpecies(std::ifstream &IN);
  inline void setNeighbors(size_t i,std::vector<int> &value);
  inline void setReaction(size_t i,BaseReaction* value);
  inline void setReaction(const std::vector<BaseReaction*> value);
  inline int addReaction( std::istream &IN );
  inline void setNeighborhood(BaseCompartmentNeighborhood *value);
  inline int addNeighborhood( std::istream &IN );
  inline void setNumNeighborhood(size_t value);
  inline void setParameter(size_t i,double value);
  
  // Other functions
  void readOrganism( std::string modelFile, std::string initFile,
		     int verbose=0 );
  void readOrganism( const char *modelFile, const char *initFile, 
		     int verbose=0 );
  void readModel(std::ifstream &IN);
  void readModel(std::string fileName);
  void readModel(const char *fileName);  
  void readInit(std::ifstream &IN);
  void readInit(std::string fileName);
  void readInit(const char *fileName);
  void initiateParameter();
  void neighborhoodCreate(std::vector< std::vector<double> > &y,
			  double t=0.0 );
  void neighborhoodUpdate(std::vector< std::vector<double> > &y,
			  double t=0.0 );
  
  void removeCompartment(size_t i);
  
  void derivs( std::vector< std::vector<double> > &y,
	       std::vector< std::vector<double> > &dydt);
  void derivsMechanical( std::vector< std::vector<double> > &y,
			 std::vector< std::vector<double> > &dydt,
			 std::vector<size_t> &mechEq,
			 std::vector<size_t> &posVar);
  void derivsTemplate( std::vector< std::vector<double> > &y,
		       std::vector< std::vector<double> > &dydt,
		       std::vector<int> &simulationFlag );
  void compartmentChange( std::vector< std::vector<double> > &y,
			  std::vector< std::vector<double> > &dydt,
			  double t=0.);
  void scaleSpace( double spaceScalingFactor);
  unsigned int findPeaksGradientAscent(const std::vector< 
				       std::vector< double > > &y,
				       size_t col, 
				       std::vector<size_t> &cellMax,
				       std::vector<size_t> &flag );
  //Printing functions
  void printParameter(std::ostream &os=std::cerr) const;
  void printVariable(std::ostream &os=std::cout) const;
  void printNeighbor(std::ostream &os=std::cout) const;
};

//! Returns the id
inline std::string Organism::id() const {
  return id_;
}

//! Returns the number of dynamical variables in each compartment
inline size_t Organism::numVariable() const {
  return numTopologyVariable()+numSpecies();
}  

//! Returns the number of compartments in the organism
inline size_t Organism::numCompartment() const {
  return compartment_.size();
}  

//! Returns the number of divisions
inline size_t Organism::numDivision() const {
  return divisionIndex_.size();
}  

//! Returns the dimension of the embedding
inline unsigned int Organism::numDimension() const {
  return topology_.numDimension();
}

//! Returns the number of different topologies (0 or 1 in current version)
inline unsigned int Organism::numTopology() const {
  return numTopology_;
}

//! Returns the number of variables in the topology
inline size_t Organism::numTopologyVariable() const {
  return topology_.numVariable();
}

//! Returns the number of species
inline size_t Organism::numSpecies() const {
  return species_.size();
}

//!Returns the number of reactions
inline size_t Organism::numReaction() const {
  return reaction_.size();
}

//! Returns the number of neighborhood rules (only zero/one allowed for...)
inline unsigned int Organism::numNeighborhood() const {
  return numNeighborhood_;
}

//! Returns the number of parameters for the complete organism
inline size_t Organism::numParameter() const {
  return parameter_.size();
}  

//!Returns a reaction (pointer)
inline BaseReaction* Organism::reaction(size_t i) const {
  return reaction_[i];
}

//!Returns the reaction pointers vector
inline const std::vector<BaseReaction*> Organism::reaction() const {
  return reaction_;
}

//! Returns the compartment with index i for const use
inline const Compartment& Organism::compartment(size_t i) const {
  return compartment_[i];
}

//! Returns the compartment with index i
inline Compartment& Organism::compartment(size_t i) {
  return compartment_[i];
}

//!Returns the compartment index of the ith division
inline int Organism::divisionIndex(size_t i) const {
  return divisionIndex_[i];
}

//!Returns the time of the ith division
inline double Organism::divisionTime(size_t i) const {
  return divisionTime_[i];
}
//!Returns the Topology for const use
inline const Topology& Organism::topology() const {
  return topology_;
}

//!Returns the Topology
inline Topology& Organism::topology() {
  return topology_;
}

//!Returns a pointer to the topology
inline Topology* Organism::topologyPointer() {
  return &topology_;
}

//!Returns the Species with index i for const use
inline const Species& Organism::species(size_t i) const {
  return species_[i];
}

//!Returns the Species with index i
inline Species& Organism::species(size_t i) {
  return species_[i];
}

//!Returns the Neighborhood for const use
inline const BaseCompartmentNeighborhood& Organism::neighborhood() const {
  return *(neighborhood_);
}

//!Returns the Neighborhood
inline BaseCompartmentNeighborhood& Organism::neighborhood() {
  return *(neighborhood_);
}

//!Returns a pointer to the neighborhood
inline BaseCompartmentNeighborhood* Organism::neighborhoodPointer() {
  return neighborhood_;
}

//!Returns a parameter value
inline double Organism::parameter(size_t i) const {
  return *(parameter_[i]);
}

//!Sets the id string
inline void Organism::setId(std::string value) {
  id_ = value;
}

//!Sets the values for the compartment with index i
inline void Organism::setCompartment(size_t i,Compartment &value) {
  compartment_[i]=value; 
}

//!Adds a compartment
inline void Organism::addCompartment(Compartment &value) {
  compartment_.push_back( value );
}

//!Adds a compartment division index and time
inline void Organism::addDivision(int indexValue,double timeValue) {
  divisionIndex_.push_back( indexValue );
  divisionTime_.push_back( timeValue );
}

//!Sets the number of topologies (0 from start and 1 when topology set)
inline void Organism::setNumTopology(unsigned int value) {
  numTopology_=value;
}

//!Sets the Topology
inline void Organism::setTopology(Topology &value) {
  std::cerr << "Organism::setTopology(Topology &value) not yet defined!\n";
  exit(0);
}

//!Sets the Species with index i
inline void Organism::setSpecies(size_t i,Species &value) {
  std::cerr << "Organism::setSpecies(size_t i,Species &value) not yet defined!\n";
  exit(0);
}

//!Adds a Species
inline void Organism::addSpecies(Species &value) {
  species_.push_back(value);
  //std::cerr << "Organism::addSpecies(Species &value) not yet defined!\n";
  //exit(0);
}

//!Adds a Species
inline void Organism::addSpecies(std::ifstream &IN) {
  species_.push_back( Species(IN) );
}

//!Sets the number of neighborhoods (0 from start and 1 when topology set)
inline void Organism::setNumNeighborhood(size_t value) {
  numNeighborhood_=value;
}

//!Adds the neighborhood
int Organism::addNeighborhood( std::istream &IN ) {
  if( !IN )
    return -1;
  neighborhood_=BaseCompartmentNeighborhood::
    createCompartmentNeighborhood(IN);
  return 0;
}

//!Sets the reaction(pointer) with index i
inline void Organism::setReaction(size_t i,BaseReaction* value) {
  reaction_[i]=value;
}

//!Sets all reaction(pointers) for a species
inline void Organism::setReaction( const std::vector<BaseReaction*> value) {
  reaction_ = value;
}

//!Sets the neighbors for compartment with index i
inline void Organism::setNeighbors(size_t i,std::vector<int> &value) {
  std::cerr << "Organism::setNeighbors(size_t i,std::vector<int> &value) "
	    << "not yet defined!\n";
  exit(0);  
}

//!Sets the parameter value
inline void Organism::setParameter(size_t i,double value) {
  *(parameter_[i])=value;
}

#endif
