#include <iostream>
#include <cstdio>
#include <cstdlib>
#include "gsl_rng.h"
#include "gsl_randist.h"

using namespace std;

const int NSIZES = 30;
const int MAXDIM = 6;
const int EDGEMASK = 0x0FFF;
const int ONEGRAIN = 0x1000;

gsl_rng *rng = gsl_rng_alloc(gsl_rng_taus);

int ndims, maxgrains, arrz, *arrzpow;
unsigned short *arr; //bits 0-B: edge, bits C-F: height

void relax(unsigned i, int *torelax, int &ntorelax);
int ilog2(unsigned a); //returns the integerpart of log2(a) if a>0, otherwise -1
void init(int argc, char **argv, int &ndims, int &arrz, 
	  int &inititer, int &maxiter, int &bufsize);
void setuparrzpow();
void setuparr();
void printarr();

int main(int argc, char **argv)
{
  gsl_rng_set(rng, 123456);

  int ntorelax, newntorelax, bufsize, bufsizesubndimsx2;
  int *torelax, *newtorelax, *temp;
  int site, i, j, maxiter, inititer;
  unsigned nrelaxed, time, sizemax;
  int nrelaxedfreq[NSIZES], timefreq[NSIZES], sizemaxfreq[NSIZES];
  for(i = 0; i < NSIZES; i++){
    nrelaxedfreq[i] = 0;
    timefreq[i] = 0;
    sizemaxfreq[i] = 0;
  }

  init(argc, argv, ndims, arrz, inititer, maxiter, bufsize);
  bufsizesubndimsx2 = bufsize-ndims*2;
  torelax = new int[bufsize];
  newtorelax = new int[bufsize];

  site = 0;

  printf("# The sandpile model in %d dimensions\n", ndims);
  printf("# n = %d, #inititerations = %d, #samplingiterations = %d\n", 
	 arrz, inititer, maxiter-inititer);
  printf("# log2(size)  nrelaxedfreq  timefreq  sizemaxfreq\n");

  for(i = 0; i < maxiter; i++){
    ntorelax = 0;
    site = gsl_rng_uniform_int(rng, arrzpow[ndims]);
    if(arr[site] >= maxgrains){
      arr[site]-=maxgrains;
      torelax[ntorelax++] = site;
    }
    else
      arr[site] += ONEGRAIN;

    for(nrelaxed = 0, time = 0, sizemax = 0; ntorelax; time++){
      if((unsigned)ntorelax > sizemax) sizemax = ntorelax;

      for(j = 0, newntorelax = 0; j < ntorelax; j++){
	relax(torelax[j], newtorelax, newntorelax);
	if(newntorelax >= bufsizesubndimsx2){
	  cerr << "Buffersize too small\n";
	  printf("# Bufferfraction %g too small: aborted\n",
		 (float)bufsize/(float)arrzpow[ndims]);
	  exit(1);
	}
      }

      temp = torelax;
      torelax = newtorelax;
      newtorelax = temp;
      nrelaxed += ntorelax;
      ntorelax = newntorelax;
    }

    if(i >= inititer){
      nrelaxedfreq[1 + ilog2(nrelaxed)]++;
      timefreq[1 + ilog2(time)]++;
      sizemaxfreq[1 + ilog2(sizemax)]++;
    }
  }

  for(i = 0; i < NSIZES; i++){
    printf("%2d %11d %11d %11d\n", i-1, 
	   nrelaxedfreq[i], timefreq[i], sizemaxfreq[i]);
  }

  delete arrzpow;
  delete arr;
  delete torelax;
  delete newtorelax;
}

void relax(unsigned i, int *torelax, int &ntorelax)
{
  static int d, site;
  if(arr[i]&EDGEMASK){
    for(d = ndims-1; d >= 0; d--){
      if((arr[i] & 1<<(d<<1))   == 0)
	if(arr[site=i-arrzpow[d]] >= maxgrains){ 
	  arr[site]-=maxgrains;
	  torelax[ntorelax++] = site;
	}
	else
	  arr[site] += ONEGRAIN;
      if((arr[i] & 1<<(d<<1)+1)   == 0)
	if(arr[site=i+arrzpow[d]] >= maxgrains){ 
	  arr[site]-=maxgrains;
	  torelax[ntorelax++] = site;
	}
	else
	  arr[site] += ONEGRAIN;
    }
  }
  else{
    for(d = ndims-1; d >= 0; d--){
      if(arr[site=i-arrzpow[d]] >= maxgrains){
	arr[site]-=maxgrains;
	torelax[ntorelax++] = site;
      }
      else
	arr[site] += ONEGRAIN;
      if(arr[site=i+arrzpow[d]] >= maxgrains){
	arr[site]-=maxgrains;
	torelax[ntorelax++] = site;
      }
      else
	arr[site] += ONEGRAIN;
    }
  }
}

int ilog2(unsigned a) //returns the integerpart of log2(a) if a>0, otherwise -1
{
  int r;
  for(r = -1; a; a>>=1, r++);
  return r;
}

void init(int argc, char **argv, int &ndims, int &arrz, 
	  int &inititer, int &maxiter, int &bufsize)
{
  if(argc != 5 && argc != 6){
    cerr << "uses: ndims size initlayers sampliter [bufferfactor]\n";
    exit(1);
  }
  ndims = atoi(argv[1]);
  if(ndims < 1 || ndims > MAXDIM){
    cerr << "The number of dimensions must be in the range 1-" << MAXDIM
	 << endl;
    exit(1);
  }
  maxgrains = ONEGRAIN*(2*ndims - 1);
  arrz = atoi(argv[2]);
  setuparrzpow();
  inititer = atoi(argv[3])*ndims*2*arrzpow[ndims];
  maxiter = atoi(argv[4]) + inititer;
  if(argc == 6)
    bufsize = (int)(atof(argv[5])*arrzpow[ndims]);
  else
    bufsize = (arrzpow[ndims]+1)/2;

  setuparr();
}

void setuparrzpow()
{
  arrzpow = new int[ndims+1];

  for(int i = 0, p = 1; i <= ndims; i++, p*=arrz)
    arrzpow[i] = p;
}

void setuparr()
{
  int d, s, k;
  arr = new unsigned short[arrzpow[ndims]];

  for(int i = arrzpow[ndims]-1; i >= 0; i--)
    arr[i] = 0;

  int i[ndims];
  for(d = ndims-2; d >= 0; d--)
    i[d] = arrz - 1;
  do{
    for(d = ndims-1; d >= 0; d--){
      s = 0;
      for(k = 0; k < d; k++)       s += i[k]*arrzpow[k];
      for(k = d+1; k < ndims; k++) s += i[k-1]*arrzpow[k];
      arr[s] |= 1<<(d<<1);
      arr[s + arrzpow[d]*(arrz-1)] |= 1<<(d<<1)+1;
    }  

    for(d = ndims-2; d >= 0 && i[d] == 0; d--)
      i[d] = arrz - 1;
    if(d >= 0) i[d]--;
  }while(d >= 0);
}

void printarr()
{
  if(ndims == 1)
    for(int c = 0; c < arrz; c++)
      cout << arr[c]/ONEGRAIN << " ";
  else
    for(int r = 0; r < arrz; r++){
      for(int c = 0; c < arrz; c++)
	cout << arr[arrz*r + c]/ONEGRAIN << " ";
      cout << endl;
    }

  cout << endl;
}
